"""Utilities with miscellaneous python extensions."""


class classproperty(property):
    """Define a class level property."""

    def __get__(self, _, owner_cls):
        return self.fget(owner_cls)



"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from keras_hub.src.models.albert.albert_tokenizer import (
    AlbertTokenizer as AlbertTokenizer,
)
from keras_hub.src.models.bart.bart_tokenizer import (
    BartTokenizer as BartTokenizer,
)
from keras_hub.src.models.bert.bert_tokenizer import (
    BertTokenizer as BertTokenizer,
)
from keras_hub.src.models.bloom.bloom_tokenizer import (
    BloomTokenizer as BloomTokenizer,
)
from keras_hub.src.models.clip.clip_tokenizer import (
    CLIPTokenizer as CLIPTokenizer,
)
from keras_hub.src.models.deberta_v3.deberta_v3_tokenizer import (
    DebertaV3Tokenizer as DebertaV3Tokenizer,
)
from keras_hub.src.models.distil_bert.distil_bert_tokenizer import (
    DistilBertTokenizer as DistilBertTokenizer,
)
from keras_hub.src.models.electra.electra_tokenizer import (
    ElectraTokenizer as ElectraTokenizer,
)
from keras_hub.src.models.f_net.f_net_tokenizer import (
    FNetTokenizer as FNetTokenizer,
)
from keras_hub.src.models.falcon.falcon_tokenizer import (
    FalconTokenizer as FalconTokenizer,
)
from keras_hub.src.models.gemma.gemma_tokenizer import (
    GemmaTokenizer as GemmaTokenizer,
)
from keras_hub.src.models.gemma3.gemma3_tokenizer import (
    Gemma3Tokenizer as Gemma3Tokenizer,
)
from keras_hub.src.models.gpt2.gpt2_tokenizer import (
    GPT2Tokenizer as GPT2Tokenizer,
)
from keras_hub.src.models.gpt_neo_x.gpt_neo_x_tokenizer import (
    GPTNeoXTokenizer as GPTNeoXTokenizer,
)
from keras_hub.src.models.llama.llama_tokenizer import (
    LlamaTokenizer as LlamaTokenizer,
)
from keras_hub.src.models.llama3.llama3_tokenizer import (
    Llama3Tokenizer as Llama3Tokenizer,
)
from keras_hub.src.models.mistral.mistral_tokenizer import (
    MistralTokenizer as MistralTokenizer,
)
from keras_hub.src.models.mixtral.mixtral_tokenizer import (
    MixtralTokenizer as MixtralTokenizer,
)
from keras_hub.src.models.moonshine.moonshine_tokenizer import (
    MoonshineTokenizer as MoonshineTokenizer,
)
from keras_hub.src.models.opt.opt_tokenizer import OPTTokenizer as OPTTokenizer
from keras_hub.src.models.pali_gemma.pali_gemma_tokenizer import (
    PaliGemmaTokenizer as PaliGemmaTokenizer,
)
from keras_hub.src.models.phi3.phi3_tokenizer import (
    Phi3Tokenizer as Phi3Tokenizer,
)
from keras_hub.src.models.qwen.qwen_tokenizer import (
    QwenTokenizer as Qwen2Tokenizer,
)
from keras_hub.src.models.qwen.qwen_tokenizer import (
    QwenTokenizer as QwenTokenizer,
)
from keras_hub.src.models.qwen_moe.qwen_moe_tokenizer import (
    QwenMoeTokenizer as QwenMoeTokenizer,
)
from keras_hub.src.models.roberta.roberta_tokenizer import (
    RobertaTokenizer as RobertaTokenizer,
)
from keras_hub.src.models.roformer_v2.roformer_v2_tokenizer import (
    RoformerV2Tokenizer as RoformerV2Tokenizer,
)
from keras_hub.src.models.siglip.siglip_tokenizer import (
    SigLIPTokenizer as SigLIPTokenizer,
)
from keras_hub.src.models.t5.t5_tokenizer import T5Tokenizer as T5Tokenizer
from keras_hub.src.models.whisper.whisper_tokenizer import (
    WhisperTokenizer as WhisperTokenizer,
)
from keras_hub.src.models.xlm_roberta.xlm_roberta_tokenizer import (
    XLMRobertaTokenizer as XLMRobertaTokenizer,
)
from keras_hub.src.tokenizers.byte_pair_tokenizer import (
    BytePairTokenizer as BytePairTokenizer,
)
from keras_hub.src.tokenizers.byte_tokenizer import (
    ByteTokenizer as ByteTokenizer,
)
from keras_hub.src.tokenizers.sentence_piece_tokenizer import (
    SentencePieceTokenizer as SentencePieceTokenizer,
)
from keras_hub.src.tokenizers.sentence_piece_tokenizer_trainer import (
    compute_sentence_piece_proto as compute_sentence_piece_proto,
)
from keras_hub.src.tokenizers.tokenizer import Tokenizer as Tokenizer
from keras_hub.src.tokenizers.unicode_codepoint_tokenizer import (
    UnicodeCodepointTokenizer as UnicodeCodepointTokenizer,
)
from keras_hub.src.tokenizers.word_piece_tokenizer import (
    WordPieceTokenizer as WordPieceTokenizer,
)
from keras_hub.src.tokenizers.word_piece_tokenizer_trainer import (
    compute_word_piece_vocabulary as compute_word_piece_vocabulary,
)



import numpy as np

from keras_hub.src.models.mobilenet.mobilenet_backbone import MobileNetBackbone

backbone_cls = MobileNetBackbone


def convert_backbone_config(timm_config):
    timm_architecture = timm_config["architecture"]

    kwargs = {
        "stackwise_num_blocks": [2, 3, 2, 3],
        "stackwise_expansion": [
            [40, 56],
            [64, 144, 144],
            [72, 72],
            [144, 288, 288],
        ],
        "stackwise_num_filters": [
            [16, 16],
            [24, 24, 24],
            [24, 24],
            [48, 48, 48],
        ],
        "stackwise_kernel_size": [[3, 3], [5, 5, 5], [5, 5], [5, 5, 5]],
        "stackwise_num_strides": [[2, 1], [2, 1, 1], [1, 1], [2, 1, 1]],
        "stackwise_se_ratio": [
            [None, None],
            [0.25, 0.25, 0.25],
            [0.25, 0.25],
            [0.25, 0.25, 0.25],
        ],
        "stackwise_activation": [
            ["relu", "relu"],
            ["hard_swish", "hard_swish", "hard_swish"],
            ["hard_swish", "hard_swish"],
            ["hard_swish", "hard_swish", "hard_swish"],
        ],
        "stackwise_padding": [[1, 1], [2, 2, 2], [2, 2], [2, 2, 2]],
        "output_num_filters": 1024,
        "input_num_filters": 16,
        "depthwise_filters": 8,
        "depthwise_stride": 2,
        "depthwise_residual": False,
        "squeeze_and_excite": 0.5,
        "last_layer_filter": 288,
        "input_activation": "relu6",
        "output_activation": "relu6",
    }

    if "mobilenetv3_" in timm_architecture:
        kwargs["input_activation"] = "hard_swish"
        kwargs["output_activation"] = "hard_swish"

    if timm_architecture == "mobilenetv3_small_050":
        pass
    elif timm_architecture == "mobilenetv3_small_100":
        modified_kwargs = {
            "stackwise_expansion": [
                [72, 88],
                [96, 240, 240],
                [120, 144],
                [288, 576, 576],
            ],
            "stackwise_num_filters": [
                [24, 24],
                [40, 40, 40],
                [48, 48],
                [96, 96, 96],
            ],
            "depthwise_filters": 16,
            "last_layer_filter": 576,
        }
        kwargs.update(modified_kwargs)
    elif timm_architecture.startswith("mobilenetv3_large_100"):
        modified_kwargs = {
            "stackwise_num_blocks": [2, 3, 4, 2, 3],
            "stackwise_expansion": [
                [64, 72],
                [72, 120, 120],
                [240, 200, 184, 184],
                [480, 672],
                [672, 960, 960],
            ],
            "stackwise_num_filters": [
                [24, 24],
                [40, 40, 40],
                [80, 80, 80, 80],
                [112, 112],
                [160, 160, 160],
            ],
            "stackwise_kernel_size": [
                [3, 3],
                [5, 5, 5],
                [3, 3, 3, 3],
                [3, 3],
                [5, 5, 5],
            ],
            "stackwise_num_strides": [
                [2, 1],
                [2, 1, 1],
                [2, 1, 1, 1],
                [1, 1],
                [2, 1, 1],
            ],
            "stackwise_se_ratio": [
                [None, None],
                [0.25, 0.25, 0.25],
                [None, None, None, None],
                [0.25, 0.25],
                [0.25, 0.25, 0.25],
            ],
            "stackwise_activation": [
                ["relu", "relu"],
                ["relu", "relu", "relu"],
                ["hard_swish", "hard_swish", "hard_swish", "hard_swish"],
                ["hard_swish", "hard_swish"],
                ["hard_swish", "hard_swish", "hard_swish"],
            ],
            "stackwise_padding": [
                [1, 1],
                [2, 2, 2],
                [1, 1, 1, 1],
                [1, 1],
                [2, 2, 2],
            ],
            "depthwise_filters": 16,
            "depthwise_stride": 1,
            "depthwise_residual": True,
            "squeeze_and_excite": None,
            "last_layer_filter": 960,
        }
        kwargs.update(modified_kwargs)
    else:
        raise ValueError(
            f"Currently, the architecture {timm_architecture} is not supported."
        )

    return kwargs


def convert_weights(backbone, loader, timm_config):
    def port_conv2d(keras_layer, hf_weight_prefix, port_bias=False):
        print(f"porting weights {hf_weight_prefix} -> {keras_layer}")
        loader.port_weight(
            keras_layer.kernel,
            hf_weight_key=f"{hf_weight_prefix}.weight",
            hook_fn=lambda x, _: np.transpose(x, (2, 3, 1, 0)),
        )

        if port_bias:
            print(f"porting bias {hf_weight_prefix} -> {keras_layer}")
            loader.port_weight(
                keras_layer.bias,
                hf_weight_key=f"{hf_weight_prefix}.bias",
            )

    def port_batch_normalization(keras_layer, hf_weight_prefix):
        print(f"porting weights {hf_weight_prefix} -> {keras_layer}")
        loader.port_weight(
            keras_layer.gamma,
            hf_weight_key=f"{hf_weight_prefix}.weight",
        )
        loader.port_weight(
            keras_layer.beta,
            hf_weight_key=f"{hf_weight_prefix}.bias",
        )
        loader.port_weight(
            keras_layer.moving_mean,
            hf_weight_key=f"{hf_weight_prefix}.running_mean",
        )
        loader.port_weight(
            keras_layer.moving_variance,
            hf_weight_key=f"{hf_weight_prefix}.running_var",
        )
        loader.port_weight(
            keras_layer.moving_variance,
            hf_weight_key=f"{hf_weight_prefix}.running_var",
        )

    # Stem
    port_conv2d(backbone.get_layer("input_conv"), "conv_stem")
    port_batch_normalization(backbone.get_layer("input_batch_norm"), "bn1")

    # DepthWise Block  (block 0)
    hf_name = "blocks.0.0"
    keras_name = "block_0_0"

    stem_block = backbone.get_layer(keras_name)

    port_conv2d(stem_block.conv1, f"{hf_name}.conv_dw")
    port_batch_normalization(stem_block.batch_normalization1, f"{hf_name}.bn1")

    if stem_block.squeeze_excite_ratio:
        stem_se_block = stem_block.se_layer
        port_conv2d(
            stem_se_block.conv_reduce, f"{hf_name}.se.conv_reduce", True
        )
        port_conv2d(
            stem_se_block.conv_expand, f"{hf_name}.se.conv_expand", True
        )

    port_conv2d(stem_block.conv2, f"{hf_name}.conv_pw")
    port_batch_normalization(stem_block.batch_normalization2, f"{hf_name}.bn2")

    # Stages
    num_stacks = len(backbone.stackwise_num_blocks)
    for block_idx in range(num_stacks):
        for inverted_block in range(backbone.stackwise_num_blocks[block_idx]):
            keras_name = f"block_{block_idx + 1}_{inverted_block}"
            hf_name = f"blocks.{block_idx + 1}.{inverted_block}"

            # Inverted Residual Block
            ir_block = backbone.get_layer(keras_name)
            port_conv2d(ir_block.conv1, f"{hf_name}.conv_pw")
            port_batch_normalization(
                ir_block.batch_normalization1, f"{hf_name}.bn1"
            )
            port_conv2d(ir_block.conv2, f"{hf_name}.conv_dw")
            port_batch_normalization(
                ir_block.batch_normalization2, f"{hf_name}.bn2"
            )

            if backbone.stackwise_se_ratio[block_idx][inverted_block]:
                ir_se_block = ir_block.squeeze_excite
                port_conv2d(
                    ir_se_block.conv_reduce,
                    f"{hf_name}.se.conv_reduce",
                    True,
                )
                port_conv2d(
                    ir_se_block.conv_expand,
                    f"{hf_name}.se.conv_expand",
                    True,
                )

            port_conv2d(ir_block.conv3, f"{hf_name}.conv_pwl")
            port_batch_normalization(
                ir_block.batch_normalization3, f"{hf_name}.bn3"
            )

    # ConvBnAct Block
    cba_block_name = f"block_{num_stacks + 1}_0"
    cba_block = backbone.get_layer(cba_block_name)
    port_conv2d(cba_block.conv, f"blocks.{num_stacks + 1}.0.conv")
    port_batch_normalization(
        cba_block.batch_normalization, f"blocks.{num_stacks + 1}.0.bn1"
    )


def convert_head(task, loader, timm_config):
    def port_conv2d(keras_layer, hf_weight_prefix, port_bias=False):
        print(f"porting weights {hf_weight_prefix} -> {keras_layer}")
        loader.port_weight(
            keras_layer.kernel,
            hf_weight_key=f"{hf_weight_prefix}.weight",
            hook_fn=lambda x, _: np.transpose(x, (2, 3, 1, 0)),
        )

        if port_bias:
            print(f"porting bias {hf_weight_prefix} -> {keras_layer}")
            loader.port_weight(
                keras_layer.bias,
                hf_weight_key=f"{hf_weight_prefix}.bias",
            )

    port_conv2d(task.output_conv, "conv_head", True)
    prefix = "classifier."
    loader.port_weight(
        task.output_dense.kernel,
        hf_weight_key=prefix + "weight",
        hook_fn=lambda x, _: np.transpose(np.squeeze(x)),
    )
    loader.port_weight(
        task.output_dense.bias,
        hf_weight_key=prefix + "bias",
    )



import math

import keras
from keras import ops


class MLP(keras.layers.Layer):
    """A MLP block with architecture.

    `input_dim -> [hidden_dim] * (num_layers - 1) -> output_dim`.

    Args:
        hidden_dim: int. The number of units in the hidden layers.
        output_dim: int. The number of units in the output layer.
        num_layers: int. The total number of dense layers to use.
        activation: str. Activation to use in the hidden layers.
            Default is `"relu"`.
    """

    def __init__(
        self, hidden_dim, output_dim, num_layers, activation="relu", **kwargs
    ):
        super().__init__(**kwargs)
        self.hidden_dim = hidden_dim
        self.output_dim = output_dim
        self.num_layers = num_layers
        self.activation = activation
        h = [hidden_dim] * (num_layers - 1)
        self.mlp_block = []
        for hidden_dim in h:
            self.mlp_block.append(
                keras.layers.Dense(hidden_dim, dtype=self.dtype_policy)
            )
            self.mlp_block.append(
                keras.layers.Activation(activation, dtype=self.dtype_policy)
            )
        self.mlp_block.append(
            keras.layers.Dense(output_dim, dtype=self.dtype_policy)
        )
        self.mlp_block = keras.models.Sequential(self.mlp_block)

    def build(self, input_shape):
        self.mlp_block.build(input_shape)
        self.built = True

    def call(self, x):
        return self.mlp_block(x)

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "hidden_dim": self.hidden_dim,
                "output_dim": self.output_dim,
                "num_layers": self.num_layers,
                "activation": self.activation,
            }
        )
        return config


class MultiHeadAttentionWithDownsampling(keras.layers.Layer):
    """Multi-Head Attention with downsampling.

    An attention layer that allows for downscaling the size of the embedding
    after projection to queries, keys, and values.
    This layer first downscales the features of input queries, keys, and
    values using a dense layer. Multi-head attention is then performed
    and the attention map is projected back (upscaled) to the number of
    input features.

    Args:
        num_heads: int. Number of attention heads.
        key_dim: int. Size of each attention head for query, key, and
            value.
        downsample_rate: int, optional. The factor by which to downscale the
            input features i.e. the input features of size `key_dim` are
            projected down to `key_dim // downsample_rate`.
    """

    def __init__(self, num_heads, key_dim, downsample_rate=1, **kwargs):
        super().__init__(**kwargs)
        self.num_heads = num_heads
        self.key_dim = key_dim
        self.downsample_rate = downsample_rate
        self.internal_dims = key_dim // downsample_rate

        # Downsample
        self.query_proj = keras.layers.Dense(
            self.internal_dims * self.num_heads, dtype=self.dtype_policy
        )
        self.key_proj = keras.layers.Dense(
            self.internal_dims * self.num_heads, dtype=self.dtype_policy
        )
        self.value_proj = keras.layers.Dense(
            self.internal_dims * self.num_heads, dtype=self.dtype_policy
        )

        # Upsample
        self.out_proj = keras.layers.Dense(
            self.key_dim * self.num_heads, dtype=self.dtype_policy
        )

    def build(self, input_shape=None):
        self.query_proj.build([None, None, self.num_heads * self.key_dim])
        self.key_proj.build([None, None, self.num_heads * self.key_dim])
        self.value_proj.build([None, None, self.num_heads * self.key_dim])
        self.out_proj.build([None, None, self.internal_dims * self.num_heads])
        self.built = True

    def _separate_heads(self, x):
        shape = ops.shape(x)
        batch_size, N, channels = shape[0], shape[1], shape[2]
        x = ops.reshape(
            x, (batch_size, N, self.num_heads, channels // self.num_heads)
        )
        return ops.transpose(x, axes=(0, 2, 1, 3))

    def _recombine_heads(self, x):
        shape = ops.shape(x)
        batch_size, num_heads, N_T, channels_per_head = (
            shape[0],
            shape[1],
            shape[2],
            shape[3],
        )
        x = ops.transpose(x, axes=(0, 2, 1, 3))
        return ops.reshape(x, (batch_size, N_T, num_heads * channels_per_head))

    def call(self, query, value, key):
        query = self.query_proj(query)
        key = self.key_proj(key)
        value = self.value_proj(value)

        # Separate into heads
        query = self._separate_heads(query)
        key = self._separate_heads(key)
        value = self._separate_heads(value)

        # Attention
        channels_per_head = ops.shape(query)[-1]
        out = ops.matmul(query, ops.transpose(key, (0, 1, 3, 2)))
        out = out / ops.sqrt(
            ops.cast(channels_per_head, dtype=self.compute_dtype)
        )
        out = ops.softmax(out, axis=-1)

        # Get output
        attention_map = out @ value
        attention_map = self._recombine_heads(attention_map)
        return self.out_proj(attention_map)

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "num_heads": self.num_heads,
                "key_dim": self.key_dim,
                "downsample_rate": self.downsample_rate,
            }
        )
        return config


class TwoWayMultiHeadAttention(keras.layers.Layer):
    """Two-way multi-head attention layer.

    Args:
        num_heads: int. Number of attention heads.
        key_dim: int. Size of each attention head for query, key, and
            value.
        intermediate_dim: int. Number of hidden dims to use in the mlp block.
        skip_first_layer_pos_embedding: bool. A boolean indicating whether to
            skip the first layer positional embeddings.
        attention_downsample_rate: int, optional. The downsample rate to use
            in the attention layers. Defaults to 2.
        activation: str, optional. The activation for the mlp block's output
            layer. Defaults to "relu".
    """

    def __init__(
        self,
        num_heads,
        key_dim,
        intermediate_dim,
        skip_first_layer_pos_embedding,
        attention_downsample_rate=2,
        activation="relu",
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.num_heads = num_heads
        self.key_dim = key_dim
        self.intermediate_dim = intermediate_dim
        self.skip_first_layer_pos_embedding = skip_first_layer_pos_embedding
        self.attention_downsample_rate = attention_downsample_rate
        self.activation = activation

        self.self_attention = MultiHeadAttentionWithDownsampling(
            num_heads=num_heads, key_dim=key_dim, dtype=self.dtype_policy
        )
        self.layer_norm1 = keras.layers.LayerNormalization(
            epsilon=1e-5, dtype=self.dtype_policy
        )
        self.cross_attention_token_to_image = (
            MultiHeadAttentionWithDownsampling(
                num_heads=num_heads,
                key_dim=key_dim,
                downsample_rate=attention_downsample_rate,
                dtype=self.dtype_policy,
            )
        )
        self.layer_norm2 = keras.layers.LayerNormalization(
            epsilon=1e-5, dtype=self.dtype_policy
        )

        self.mlp_block = MLP(
            intermediate_dim,
            key_dim * num_heads,
            num_layers=2,
            activation=activation,
            dtype=self.dtype_policy,
        )

        self.layer_norm3 = keras.layers.LayerNormalization(
            epsilon=1e-5, dtype=self.dtype_policy
        )
        self.cross_attention_image_to_token = (
            MultiHeadAttentionWithDownsampling(
                num_heads=num_heads,
                key_dim=key_dim,
                downsample_rate=attention_downsample_rate,
                dtype=self.dtype_policy,
            )
        )
        self.layer_norm4 = keras.layers.LayerNormalization(
            epsilon=1e-5, dtype=self.dtype_policy
        )

    def build(self, input_shape=None):
        self.self_attention.build()
        self.layer_norm1.build([None, None, self.num_heads * self.key_dim])
        self.cross_attention_token_to_image.build()
        self.layer_norm2.build([None, None, self.num_heads * self.key_dim])
        self.mlp_block.build([None, None, self.num_heads * self.key_dim])
        self.layer_norm3.build([None, None, self.num_heads * self.key_dim])
        self.cross_attention_image_to_token.build()
        self.layer_norm4.build([None, None, self.num_heads * self.key_dim])
        self.built = True

    def call(self, queries, keys, query_pos_embedding, key_pos_embedding):
        if self.skip_first_layer_pos_embedding:
            queries = self.self_attention(
                query=queries, value=queries, key=queries
            )
        else:
            queries_with_pos_embedding = queries + query_pos_embedding
            attention_map = self.self_attention(
                query=queries_with_pos_embedding,
                key=queries_with_pos_embedding,
                value=queries,
            )
            queries = queries + attention_map
        queries = self.layer_norm1(queries)

        queries_with_pos_embedding = queries + query_pos_embedding
        keys_with_pos_embedding = keys + key_pos_embedding
        attention_map = self.cross_attention_token_to_image(
            query=queries_with_pos_embedding,
            key=keys_with_pos_embedding,
            value=keys,
        )
        queries = queries + attention_map
        queries = self.layer_norm2(queries)

        mlp_out = self.mlp_block(queries)
        queries = queries + mlp_out
        queries = self.layer_norm3(queries)

        queries_with_pos_embedding = queries + query_pos_embedding
        keys_with_pos_embedding = keys + key_pos_embedding
        attention_map = self.cross_attention_image_to_token(
            query=keys_with_pos_embedding,
            key=queries_with_pos_embedding,
            value=queries,
        )
        keys = keys + attention_map
        keys = self.layer_norm4(keys)

        return queries, keys

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "num_heads": self.num_heads,
                "key_dim": self.key_dim,
                "intermediate_dim": self.intermediate_dim,
                "skip_first_layer_pos_embedding": (
                    self.skip_first_layer_pos_embedding
                ),
                "attention_downsample_rate": self.attention_downsample_rate,
                "activation": self.activation,
            }
        )
        return config


class RandomFrequencyPositionalEmbeddings(keras.layers.Layer):
    """Positional encoding using random spatial frequencies.

    This layer maps coordinates/points in 2D space to positional
    encodings using random spatial frequencies.

    Args:
        num_positional_features: int. Number of positional features
            in the output.
        scale: float. The standard deviation of the random frequencies.
    """

    def __init__(self, num_positional_features, scale, **kwargs):
        super().__init__(**kwargs)
        self.num_positional_features = num_positional_features
        self.scale = scale
        self.positional_encoding_gaussian_matrix = self.add_weight(
            name="positional_encoding_gaussian_matrix",
            shape=(2, self.num_positional_features),
            dtype=self.variable_dtype,
            trainable=False,
            initializer=keras.initializers.get("normal"),
        )

    def build(self, input_shape=None):
        self.built = True

    def _positional_encodings(self, coords):
        coords = coords * 2 - 1
        coords = coords @ ops.cast(
            self.positional_encoding_gaussian_matrix, dtype=self.compute_dtype
        )
        coords = coords * (2 * math.pi)
        return ops.concatenate([ops.sin(coords), ops.cos(coords)], axis=-1)

    def call(self, size):
        return self.encode_image(size)

    def encode_image(self, size):
        """Generate a positional encoding for an image of any given size.
        Args:
            size: tuple[int, int]. The size of the image.
        Returns:
            tensor: Positional encoding of the image.
        """
        height, width = size
        grid = ops.ones(shape=(height, width), dtype=self.compute_dtype)
        y_embed = ops.cumsum(grid, axis=0) - 0.5
        x_embed = ops.cumsum(grid, axis=1) - 0.5
        y_embed = y_embed / ops.cast(height, self.compute_dtype)
        x_embed = x_embed / ops.cast(width, self.compute_dtype)
        return self._positional_encodings(
            ops.stack([x_embed, y_embed], axis=-1)
        )

    def encode_coordinates(self, coords_input, image_size):
        """Positionally encode points that are not normalized to `[0, 1]`.
        Args:
            coords_input: tensor. 2D coordinates/points to map.
            image_size: tuple[int, int]. Height and width of the image
                being prompted.
        Returns:
            tensor: Positional encodings of the normalized coordinates.
        """
        coords_normalized = ops.stack(
            [
                coords_input[..., 0] / image_size[1],
                coords_input[..., 1] / image_size[0],
            ],
            axis=-1,
        )
        return self._positional_encodings(coords_normalized)

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "num_positional_features": self.num_positional_features,
                "scale": self.scale,
            }
        )
        return config



import keras
from keras import layers

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.backbone import Backbone


@keras_hub_export("keras_hub.models.VGGBackbone")
class VGGBackbone(Backbone):
    """This class represents Keras Backbone of VGG model.

    This class implements a VGG backbone as described in [Very Deep
    Convolutional Networks for Large-Scale Image Recognition](
    https://arxiv.org/abs/1409.1556)(ICLR 2015).

    Args:
      stackwise_num_repeats: list of ints, number of repeated convolutional
            blocks per VGG block. For VGG16 this is [2, 2, 3, 3, 3] and for
            VGG19 this is [2, 2, 4, 4, 4].
      stackwise_num_filters: list of ints, filter size for convolutional
            blocks per VGG block. For both VGG16 and VGG19 this is [
            64, 128, 256, 512, 512].
      image_shape: tuple, optional shape tuple, defaults to (None, None, 3).

    Examples:
    ```python
    input_data = np.ones((2, 224, 224, 3), dtype="float32")

    # Pretrained VGG backbone.
    model = keras_hub.models.VGGBackbone.from_preset("vgg_16_imagenet")
    model(input_data)

    # Randomly initialized VGG backbone with a custom config.
    model = keras_hub.models.VGGBackbone(
        stackwise_num_repeats = [2, 2, 3, 3, 3],
        stackwise_num_filters = [64, 128, 256, 512, 512],
        image_shape = (224, 224, 3),
    )
    model(input_data)
    ```
    """

    def __init__(
        self,
        stackwise_num_repeats,
        stackwise_num_filters,
        image_shape=(None, None, 3),
        **kwargs,
    ):
        # === Functional Model ===
        img_input = keras.layers.Input(shape=image_shape)
        x = img_input

        for stack_index in range(len(stackwise_num_repeats)):
            x = apply_vgg_block(
                x=x,
                num_layers=stackwise_num_repeats[stack_index],
                filters=stackwise_num_filters[stack_index],
                kernel_size=(3, 3),
                activation="relu",
                padding="same",
                max_pool=True,
                name=f"block{stack_index + 1}",
            )

        super().__init__(inputs=img_input, outputs=x, **kwargs)

        # === Config ===
        self.stackwise_num_repeats = stackwise_num_repeats
        self.stackwise_num_filters = stackwise_num_filters
        self.image_shape = image_shape

    def get_config(self):
        return {
            "stackwise_num_repeats": self.stackwise_num_repeats,
            "stackwise_num_filters": self.stackwise_num_filters,
            "image_shape": self.image_shape,
        }


def apply_vgg_block(
    x,
    num_layers,
    filters,
    kernel_size,
    activation,
    padding,
    max_pool,
    name,
):
    """
    Applies VGG block
    Args:
        x: Tensor, input tensor to pass through network
        num_layers: int, number of CNN layers in the block
        filters: int, filter size of each CNN layer in block
        kernel_size: int (or) tuple, kernel size for CNN layer in block
        activation: str (or) callable, activation function for each CNN layer in
            block
        padding: str (or) callable, padding function for each CNN layer in block
        max_pool: bool, whether to add MaxPooling2D layer at end of block
        name: str, name of the block

    Returns:
        keras.KerasTensor
    """
    for num in range(1, num_layers + 1):
        x = layers.Conv2D(
            filters,
            kernel_size,
            activation=activation,
            padding=padding,
            name=f"{name}_conv{num}",
        )(x)
    if max_pool:
        x = layers.MaxPooling2D((2, 2), (2, 2), name=f"{name}_pool")(x)
    return x



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.causal_lm_preprocessor import CausalLMPreprocessor
from keras_hub.src.models.qwen_moe.qwen_moe_backbone import QwenMoeBackbone
from keras_hub.src.models.qwen_moe.qwen_moe_tokenizer import QwenMoeTokenizer


@keras_hub_export(
    [
        "keras_hub.models.QwenMoeCausalLMPreprocessor",
    ]
)
class QwenMoeCausalLMPreprocessor(CausalLMPreprocessor):
    backbone_cls = QwenMoeBackbone
    tokenizer_cls = QwenMoeTokenizer

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)



"""DeBERTa model preset configurations."""

backbone_presets = {
    "deberta_v3_extra_small_en": {
        "metadata": {
            "description": (
                "12-layer DeBERTaV3 model where case is maintained. "
                "Trained on English Wikipedia, BookCorpus and OpenWebText."
            ),
            "params": 70682112,
            "path": "deberta_v3",
        },
        "kaggle_handle": "kaggle://keras/deberta_v3/keras/deberta_v3_extra_small_en/3",
    },
    "deberta_v3_small_en": {
        "metadata": {
            "description": (
                "6-layer DeBERTaV3 model where case is maintained. "
                "Trained on English Wikipedia, BookCorpus and OpenWebText."
            ),
            "params": 141304320,
            "path": "deberta_v3",
        },
        "kaggle_handle": "kaggle://keras/deberta_v3/keras/deberta_v3_small_en/3",
    },
    "deberta_v3_base_en": {
        "metadata": {
            "description": (
                "12-layer DeBERTaV3 model where case is maintained. "
                "Trained on English Wikipedia, BookCorpus and OpenWebText."
            ),
            "params": 183831552,
            "path": "deberta_v3",
        },
        "kaggle_handle": "kaggle://keras/deberta_v3/keras/deberta_v3_base_en/3",
    },
    "deberta_v3_large_en": {
        "metadata": {
            "description": (
                "24-layer DeBERTaV3 model where case is maintained. "
                "Trained on English Wikipedia, BookCorpus and OpenWebText."
            ),
            "params": 434012160,
            "path": "deberta_v3",
        },
        "kaggle_handle": "kaggle://keras/deberta_v3/keras/deberta_v3_large_en/3",
    },
    "deberta_v3_base_multi": {
        "metadata": {
            "description": (
                "12-layer DeBERTaV3 model where case is maintained. "
                "Trained on the 2.5TB multilingual CC100 dataset."
            ),
            "params": 278218752,
            "path": "deberta_v3",
        },
        "kaggle_handle": "kaggle://keras/deberta_v3/keras/deberta_v3_base_multi/3",
    },
}



from keras_hub.src.models.llama3.llama3_backbone import Llama3Backbone
from keras_hub.src.models.llama3.llama3_presets import backbone_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, Llama3Backbone)



import keras

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.preprocessor import Preprocessor
from keras_hub.src.utils.tensor_utils import preprocessing_function


@keras_hub_export(
    [
        "keras_hub.models.ObjectDetectorPreprocessor",
        "keras_hub.models.ImageObjectDetectorPreprocessor",
    ]
)
class ObjectDetectorPreprocessor(Preprocessor):
    """Base class for object detector preprocessing layers.

    `ObjectDetectorPreprocessor` tasks wraps a
    `keras_hub.layers.Preprocessor` to create a preprocessing layer for
    object detection tasks. It is intended to be paired with a
    `keras_hub.models.ImageObjectDetector` task.

    All `ObjectDetectorPreprocessor` take three inputs, `x`, `y`, and
    `sample_weight`. `x`, the first input, should always be included. It can
    be a image or batch of images. See examples below. `y` and `sample_weight`
    are optional inputs that will be passed through unaltered. Usually, `y`
    will be the a dict of `{"boxes": Tensor(batch_size, num_boxes, 4),
    "classes": (batch_size, num_boxes)}.

    The layer will returns either `x`, an `(x, y)` tuple if labels were
    provided, or an `(x, y, sample_weight)` tuple if labels and sample weight
    were provided. `x` will be the input images after all model preprocessing
    has been applied.

    All `ObjectDetectorPreprocessor` tasks include a `from_preset()`
    constructor which can be used to load a pre-trained config and
    vocabularies. You can call the `from_preset()` constructor directly on
    this base class, in which case the correct class for your model will be
    automatically instantiated.

    Args:
        image_converter: Preprocessing pipeline for images.

    Examples.
    ```python
    preprocessor = keras_hub.models.ObjectDetectorPreprocessor.from_preset(
        "retinanet_resnet50",
    )
    """

    def __init__(
        self,
        image_converter=None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.image_converter = image_converter

    @preprocessing_function
    def call(self, x, y=None, sample_weight=None):
        if y is None:
            x = self.image_converter(x)
        else:
            # Pass bounding boxes through image converter in the dictionary
            # with keys format standardized by core Keras.
            output = self.image_converter(
                {
                    "images": x,
                    "bounding_boxes": y,
                }
            )
            x = output["images"]
            y = output["bounding_boxes"]
        return keras.utils.pack_x_y_sample_weight(x, y, sample_weight)



import keras
from keras import ops


# TODO: Deprecate this in favor of
# `keras.layers.LayerNormalization(rms_scaling=True)` once Keras 2 support is
# removed.
class LlamaLayerNorm(keras.layers.Layer):
    """A normalization layer for Llama that implements RMS normalization."""

    def __init__(self, epsilon=1e-6, **kwargs):
        super().__init__(**kwargs)
        self.epsilon = epsilon

    def build(self, input_shape):
        dim = input_shape[-1]
        self.scale = self.add_weight(
            name="scale",
            trainable=True,
            shape=(dim,),
            initializer="ones",
            dtype=self.variable_dtype,
        )
        self.built = True

    def call(self, x):
        x = ops.cast(x, "float32")
        var = ops.mean(ops.power(x, 2), axis=-1, keepdims=True)
        x = x * ops.rsqrt(var + self.epsilon)
        return ops.cast(x * self.scale, self.compute_dtype)

    def get_config(self):
        config = super().get_config()
        config.update({"epsilon": self.epsilon})
        return config



import keras

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.backbone import Backbone
from keras_hub.src.models.vit.vit_layers import ViTEncoder
from keras_hub.src.models.vit.vit_layers import ViTPatchingAndEmbedding
from keras_hub.src.utils.keras_utils import standardize_data_format


@keras_hub_export("keras_hub.models.ViTBackbone")
class ViTBackbone(Backbone):
    """Vision Transformer (ViT) backbone.

    This backbone implements the Vision Transformer architecture as described in
    [An Image is Worth 16x16 Words: Transformers for Image Recognition at Scale](https://arxiv.org/abs/2010.11929).
    It transforms the input image into a sequence of patches, embeds them, and
    then processes them through a series of Transformer encoder layers.

    Args:
        image_shape: A tuple or list of 3 integers representing the shape of the
            input image `(height, width, channels)`, `height` and `width` must
            be equal.
        patch_size: int. The size of each image patch, the input image will be
            divided into patches of shape `(patch_size, patch_size)`.
        num_layers: int. The number of transformer encoder layers.
        num_heads: int. specifying the number of attention heads in each
            Transformer encoder layer.
        hidden_dim: int. The dimensionality of the hidden representations.
        mlp_dim: int. The dimensionality of the intermediate MLP layer in
            each Transformer encoder layer.
        dropout_rate: float. The dropout rate for the Transformer encoder
            layers.
        attention_dropout: float. The dropout rate for the attention mechanism
            in each Transformer encoder layer.
        layer_norm_epsilon: float. Value used for numerical stability in
            layer normalization.
        use_mha_bias: bool. Whether to use bias in the multi-head
            attention layers.
        use_mlp_bias: bool. Whether to use bias in the MLP layers.
        data_format: str.  `"channels_last"` or `"channels_first"`, specifying
            the data format for the input image. If `None`, defaults to
            `"channels_last"`.
        dtype: The dtype of the layer weights. Defaults to None.
        **kwargs: Additional keyword arguments to be passed to the parent
            `Backbone` class.
    """

    def __init__(
        self,
        image_shape,
        patch_size,
        num_layers,
        num_heads,
        hidden_dim,
        mlp_dim,
        dropout_rate=0.0,
        attention_dropout=0.0,
        layer_norm_epsilon=1e-6,
        use_mha_bias=True,
        use_mlp_bias=True,
        data_format=None,
        dtype=None,
        **kwargs,
    ):
        # === Laters ===
        data_format = standardize_data_format(data_format)
        h_axis, w_axis, channels_axis = (
            (-3, -2, -1) if data_format == "channels_last" else (-2, -1, -3)
        )
        # Check that the input image is well specified.
        if image_shape[h_axis] is None or image_shape[w_axis] is None:
            raise ValueError(
                f"Image shape must have defined height and width. Found `None` "
                f"at index {h_axis} (height) or {w_axis} (width). "
                f"Image shape: {image_shape}"
            )
        if image_shape[h_axis] != image_shape[w_axis]:
            raise ValueError(
                f"Image height and width must be equal. Found height: "
                f"{image_shape[h_axis]}, width: {image_shape[w_axis]} at "
                f"indices {h_axis} and {w_axis} respectively. Image shape: "
                f"{image_shape}"
            )

        num_channels = image_shape[channels_axis]

        # === Functional Model ===
        inputs = keras.layers.Input(shape=image_shape)

        x = ViTPatchingAndEmbedding(
            image_size=image_shape[h_axis],
            patch_size=patch_size,
            hidden_dim=hidden_dim,
            num_channels=num_channels,
            data_format=data_format,
            dtype=dtype,
            name="vit_patching_and_embedding",
        )(inputs)

        output = ViTEncoder(
            num_layers=num_layers,
            num_heads=num_heads,
            hidden_dim=hidden_dim,
            mlp_dim=mlp_dim,
            dropout_rate=dropout_rate,
            attention_dropout=attention_dropout,
            layer_norm_epsilon=layer_norm_epsilon,
            use_mha_bias=use_mha_bias,
            use_mlp_bias=use_mlp_bias,
            dtype=dtype,
            name="vit_encoder",
        )(x)

        super().__init__(
            inputs=inputs,
            outputs=output,
            dtype=dtype,
            **kwargs,
        )

        # === Config ===
        self.image_shape = image_shape
        self.patch_size = patch_size
        self.num_layers = num_layers
        self.num_heads = num_heads
        self.hidden_dim = hidden_dim
        self.mlp_dim = mlp_dim
        self.dropout_rate = dropout_rate
        self.attention_dropout = attention_dropout
        self.layer_norm_epsilon = layer_norm_epsilon
        self.use_mha_bias = use_mha_bias
        self.use_mlp_bias = use_mlp_bias
        self.data_format = data_format

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "image_shape": self.image_shape,
                "patch_size": self.patch_size,
                "num_layers": self.num_layers,
                "num_heads": self.num_heads,
                "hidden_dim": self.hidden_dim,
                "mlp_dim": self.mlp_dim,
                "dropout_rate": self.dropout_rate,
                "attention_dropout": self.attention_dropout,
                "layer_norm_epsilon": self.layer_norm_epsilon,
                "use_mha_bias": self.use_mha_bias,
                "use_mlp_bias": self.use_mlp_bias,
            }
        )
        return config



import keras
from keras import ops

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.gemma.rms_normalization import RMSNormalization
from keras_hub.src.utils.keras_utils import clone_initializer


@keras_hub_export("keras_hub.models.Gemma3VisionEncoder")
class Gemma3VisionEncoder(keras.Model):
    """Vision Transformer (ViT) model for Gemma3.

    Args:
        image_size: int. The height/width of the image. Both height and width is
            expected to be the same.
        patch_size: int. The size of each square patch in the input image.
        num_heads: int. The number of attention heads for the vision(image)
            transformer encoder.
        hidden_dim: int. The size of the transformer hidden state at the end
            of each vision transformer layer.
        num_layers: int. The number of transformer layers.
        intermediate_dim: int. The output dimension of the first Dense layer in
            a two-layer feedforward network for transformer.
        output_dim: int. The odimension of the output returned by the model.
        pool_size: int. Factors by which to downscale `(dim1, dim2)` in the
            average pooling layer. The same value is used for `"strides"`.
            Defaults to 14.
        layer_norm_epsilon: float. The epsilon value user for every layer norm
            in all transformer blocks. Defaults to `1e-6`.
        dtype: string or `keras.mixed_precision.DTypePolicy`. The dtype to use
            for the models computations and weights. Note that some
            computations, such as softmax and layer normalization will always
            be done a float32 precision regardless of dtype.

    Example:
    ```python
    image = np.random.rand(224, 224, 3)
    vit_model = Gemma3VisionEncoder(image_size=224)
    # The output will be of shape:
    # [batch_size, num_vision_tokens_per_image, hidden_dim]
    output = vit_model([image])
    ```
    """

    def __init__(
        self,
        image_size,
        patch_size,
        num_heads,
        hidden_dim,
        num_layers,
        intermediate_dim,
        output_dim,
        pool_size=14,
        layer_norm_epsilon=1e-6,
        dtype=None,
        **kwargs,
    ):
        # If the passed dtype is `bfloat16`, use `float32` to maintain parity
        # with other framework implementations.
        if dtype == "bfloat16":
            dtype = "float32"

        # === Functional Model ===
        image_input = keras.Input(
            shape=(None, image_size, image_size, 3),
            name="images",
        )
        x = image_input  # Intermediate result.
        x = Gemma3VisionEncoderBlock(
            hidden_dim=hidden_dim,
            num_layers=num_layers,
            num_heads=num_heads,
            intermediate_dim=intermediate_dim,
            patch_size=patch_size,
            image_size=image_size,
            dtype=dtype,
            name="image_encoder",
        )(x)

        x = Gemma3VisionAveragePooling(
            image_size=image_size,
            patch_size=patch_size,
            pool_size=pool_size,
            dtype=dtype,
            name="pooling",
        )(x)

        x = Gemma3VisionOutput(
            output_dim=output_dim,
            layer_norm_epsilon=layer_norm_epsilon,
            kernel_initializer=keras.initializers.RandomNormal(
                mean=0.0, stddev=0.01
            ),
            dtype=dtype,
            name="vision_output_encoder",
        )(x)

        outputs = x
        super().__init__(
            inputs=image_input,
            outputs=outputs,
            **kwargs,
        )

        # === Config ===
        self.image_size = image_size
        self.patch_size = patch_size
        self.num_heads = num_heads
        self.hidden_dim = hidden_dim
        self.num_layers = num_layers
        self.intermediate_dim = intermediate_dim
        self.output_dim = output_dim
        self.pool_size = pool_size
        self.layer_norm_epsilon = layer_norm_epsilon
        self.num_vision_tokens_per_image = (
            (image_size // patch_size) ** 2
        ) // (pool_size**2)

        # Before Keras 3.2, there is no `keras.dtype_policies.get`.
        if hasattr(keras.dtype_policies, "get"):
            self.dtype_policy = keras.dtype_policies.get(dtype)
        else:
            if isinstance(dtype, keras.dtype_policies.DTypePolicy):
                dtype = dtype.name
            dtype = dtype or keras.config.dtype_policy().name
            self.dtype_policy = keras.dtype_policies.DTypePolicy(dtype)

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "num_heads": self.num_heads,
                "hidden_dim": self.hidden_dim,
                "num_layers": self.num_layers,
                "intermediate_dim": self.intermediate_dim,
                "output_dim": self.output_dim,
                "pool_size": self.pool_size,
                "image_size": self.image_size,
                "patch_size": self.patch_size,
                "layer_norm_epsilon": self.layer_norm_epsilon,
            }
        )
        return config


class Gemma3VisionEmbedding(keras.layers.Layer):
    def __init__(
        self,
        image_size,
        patch_size,
        hidden_dim,
        num_channels=3,
        dtype=None,
        **kwargs,
    ):
        super().__init__(dtype=dtype, **kwargs)
        self.hidden_dim = hidden_dim
        self.image_size = image_size
        self.patch_size = patch_size
        self.num_channels = num_channels
        self.patch_embedding = keras.layers.Conv2D(
            filters=self.hidden_dim,
            kernel_size=self.patch_size,
            strides=self.patch_size,
            padding="valid",
            activation=None,
            dtype=dtype,
            name="embedding_conv",
        )
        self.num_patches = (self.image_size // self.patch_size) ** 2
        self.num_positions = self.num_patches
        self.position_embedding = keras.layers.Embedding(
            self.num_positions,
            self.hidden_dim,
            dtype=dtype,
            name="position_embedding",
        )

        self.position_ids = ops.expand_dims(
            ops.arange(self.num_positions), axis=0
        )

    def build(self, input_shape):
        self.patch_embedding.build(input_shape)
        self.position_embedding.build([1, self.num_positions])
        self.built = True

    def call(self, input_tokens):
        x = self.patch_embedding(input_tokens)
        input_shape = ops.shape(x)
        x = ops.reshape(x, [input_shape[0], self.num_patches, self.hidden_dim])
        x = x + self.position_embedding(self.position_ids)
        return x

    def compute_output_shape(self, input_shape):
        return (
            input_shape[0],
            self.num_patches,
            self.hidden_dim,
        )


class Gemma3VisionAttention(keras.layers.Layer):
    """
    Adapted from https://github.com/huggingface/transformers/blob/main/src/transformers/models/clip/modeling_clip.py
    """

    def __init__(
        self,
        hidden_dim,
        num_heads,
        dropout=0.0,
        dtype=None,
        **kwargs,
    ):
        super().__init__(dtype=dtype, **kwargs)

        self.hidden_dim = hidden_dim
        self.num_heads = num_heads
        self.dropout = dropout
        self.head_dim = self.hidden_dim // self.num_heads
        if self.head_dim * self.num_heads != self.hidden_dim:
            raise ValueError(
                f"hidden_dim must be divisible by num_heads (got `hidden_dim`"
                f": {self.hidden_dim} and `num_heads`:"
                f" {self.num_heads})."
            )
        self.dropout_layer = keras.layers.Dropout(
            self.dropout,
            dtype=dtype,
            name="dropout",
        )
        self.scale = self.head_dim**-0.5
        self.query_proj = keras.layers.Dense(
            units=self.hidden_dim,
            dtype=dtype,
            name="query_proj",
        )
        self.key_proj = keras.layers.Dense(
            units=self.hidden_dim,
            dtype=dtype,
            name="key_proj",
        )
        self.value_proj = keras.layers.Dense(
            units=self.hidden_dim,
            dtype=dtype,
            name="value_proj",
        )
        self.out_proj = keras.layers.Dense(
            units=self.hidden_dim,
            dtype=dtype,
            name="out_proj",
        )

    def build(self, input_shape):
        self.query_proj.build([None, None, self.hidden_dim])
        self.key_proj.build([None, None, self.hidden_dim])
        self.value_proj.build([None, None, self.hidden_dim])
        self.out_proj.build([None, None, self.hidden_dim])
        self.built = True

    def _transpose_for_scores(self, tensor, batch_size):
        """
        Adapted from https://github.com/huggingface/transformers/blob/8e164c5400b7b413c7b8fb32e35132001effc970/src/transformers/models/bert/modeling_tf_bert.py#L252
        """
        # [batch_size, seq_len, all_head_dim] ->
        # [batch_size, seq_len, num_heads, head_dim]
        seq_len = ops.shape(tensor)[1]
        tensor = ops.reshape(
            tensor, (batch_size, seq_len, self.num_heads, self.head_dim)
        )
        # [batch_size, seq_len, num_heads, head_dim] ->
        # [batch_size, num_heads, seq_len, head_dim]
        return ops.transpose(tensor, axes=[0, 2, 1, 3])

    def call(
        self,
        x,
        attention_mask=None,
        return_attention_scores=None,
        training=False,
    ):
        batch_size = ops.shape(x)[0]
        mixed_query_layer = self.query_proj(inputs=x)
        mixed_key_layer = self.key_proj(inputs=x)
        mixed_value_layer = self.value_proj(inputs=x)
        query_layer = self._transpose_for_scores(mixed_query_layer, batch_size)
        key_layer = self._transpose_for_scores(mixed_key_layer, batch_size)
        value_layer = self._transpose_for_scores(mixed_value_layer, batch_size)

        # Scaled dot product between key and query = raw attention scores.
        attention_scores = ops.matmul(
            query_layer, ops.transpose(key_layer, axes=[0, 1, 3, 2])
        )
        dk = ops.cast(ops.sqrt(self.head_dim), dtype=attention_scores.dtype)
        attention_scores = ops.divide(
            attention_scores, dk
        )  # (batch_size, num_heads, seq_len_q, seq_len_k)

        if attention_mask is not None:
            # Apply the attention mask (precomputed for all layers in the
            # call() function)
            attention_scores = ops.add(attention_scores, attention_mask)

        # Normalize the attention scores to probabilities.
        attention_probs = ops.softmax(attention_scores, axis=-1)

        # This is actually dropping out entire tokens to attend to, which might
        # seem a bit unusual, but is taken from the original Transformer paper.
        dropout_attention_probs = self.dropout_layer(
            inputs=attention_probs, training=training
        )

        attn_output = ops.matmul(dropout_attention_probs, value_layer)
        attn_output = ops.transpose(attn_output, axes=[0, 2, 1, 3])

        # (batch_size, seq_len_q, hidden_dim)
        seq_len_q = ops.shape(attn_output)[1]
        attn_output = ops.reshape(
            attn_output, (batch_size, seq_len_q, self.hidden_dim)
        )

        attn_output = self.out_proj(attn_output, training=training)
        return (attn_output, attention_probs)

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "hidden_dim": self.hidden_dim,
                "num_heads": self.num_heads,
                "dropout": self.dropout,
            }
        )
        return config


class Gemma3VisionEncoderLayer(keras.layers.Layer):
    def __init__(
        self,
        num_heads,
        intermediate_dim,
        layer_norm_epsilon=1e-6,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.num_heads = num_heads
        self.intermediate_dim = intermediate_dim
        self.layer_norm_epsilon = layer_norm_epsilon

    def compute_attention(self, x, mask=None):
        if mask is not None:
            mask = ops.cast(mask, dtype=x.dtype)
        return self.attn(x, attention_mask=mask)[0]

    def build(self, input_shape):
        hidden_dim = input_shape[-1]
        self.attn = Gemma3VisionAttention(
            hidden_dim,
            self.num_heads,
            dtype=self.dtype_policy,
            name="multi_head_attention",
        )
        self.layer_norm_1 = keras.layers.LayerNormalization(
            epsilon=self.layer_norm_epsilon,
            dtype=self.dtype_policy,
            name="layer_norm_1",
        )
        self.mlp_dense_1 = keras.layers.Dense(
            self.intermediate_dim,
            dtype=self.dtype_policy,
            name="mlp_dense_1",
        )
        self.mlp_dense_2 = keras.layers.Dense(
            hidden_dim,
            dtype=self.dtype_policy,
            name="mlp_dense_2",
        )
        self.layer_norm_2 = keras.layers.LayerNormalization(
            epsilon=self.layer_norm_epsilon,
            dtype=self.dtype_policy,
            name="layer_norm_2",
        )
        self.attn.build(None)
        self.layer_norm_1.build([None, None, hidden_dim])
        self.mlp_dense_1.build([None, None, hidden_dim])
        self.mlp_dense_2.build([None, None, self.intermediate_dim])
        self.layer_norm_2.build([None, None, hidden_dim])
        self.built = True

    def call(self, x, mask=None):
        residual = x
        x = self.layer_norm_1(x)
        # mask = ops.ones_like(x) if mask is None else mask
        x = self.compute_attention(x, mask)
        x = x + residual
        residual = x
        x = self.mlp_dense_1(self.layer_norm_2(residual))
        x = keras.activations.gelu(x, approximate=True)
        x = self.mlp_dense_2(x)
        return residual + x

    def compute_output_shape(self, inputs_shape):
        return inputs_shape

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "num_heads": self.num_heads,
                "intermediate_dim": self.intermediate_dim,
                "layer_norm_epsilon": self.layer_norm_epsilon,
            }
        )
        return config


class Gemma3VisionEncoderBlock(keras.layers.Layer):
    def __init__(
        self,
        patch_size,
        image_size,
        hidden_dim,
        num_layers,
        num_heads,
        intermediate_dim,
        layer_norm_epsilon=1e-6,
        dtype=None,
        **kwargs,
    ):
        super().__init__(dtype=dtype, **kwargs)
        self.patch_size = patch_size
        self.image_size = image_size
        self.hidden_dim = hidden_dim
        self.num_layers = num_layers
        self.num_heads = num_heads
        self.intermediate_dim = intermediate_dim
        self.layer_norm_epsilon = layer_norm_epsilon
        self.encoder_layer_norm = keras.layers.LayerNormalization(
            epsilon=layer_norm_epsilon,
            dtype=dtype,
            name="encoder_layer_norm",
        )
        self.vision_embeddings = Gemma3VisionEmbedding(
            hidden_dim=hidden_dim,
            patch_size=patch_size,
            image_size=image_size,
            dtype=dtype,
            name="encoder_embeddings",
        )
        self.resblocks = [
            Gemma3VisionEncoderLayer(
                self.num_heads,
                self.intermediate_dim,
                dtype=dtype,
                name=f"encoder_block_{i}",
            )
            for i in range(self.num_layers)
        ]

    def build(self, inputs_shape):
        # Collapse `batch_size`, dummy axis, `max_images_per_prompt` into one.
        inputs_shape = [None] + list(inputs_shape[2:])
        self.vision_embeddings.build(inputs_shape)
        for block in self.resblocks:
            block.build([None, None, self.hidden_dim])
        self.encoder_layer_norm.build([None, None, self.hidden_dim])
        self.built = True

    def call(self, inputs, mask=None):
        inputs_shape = ops.shape(inputs)

        # Collapse `batch_size`, dummy axis, `max_images_per_prompt` into one.
        inputs = ops.reshape(
            inputs,
            [inputs_shape[0] * inputs_shape[1]] + list(inputs_shape[2:]),
        )

        x = self.vision_embeddings(inputs)
        for block in self.resblocks:
            x = block(x, mask=mask)
        x = self.encoder_layer_norm(x)
        return x

    def compute_output_shape(self, inputs_shape):
        if inputs_shape is None:
            # Fix the compatibility issue with Keras 3.1 where
            # `compute_output_spec` fails to propagate `inputs_shape`
            # correctly, causing it to be `None`.
            return [None, None, self.hidden_dim]
        return [
            None,
            (inputs_shape[2] // self.patch_size) ** 2,
            self.hidden_dim,
        ]

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "hidden_dim": self.hidden_dim,
                "num_layers": self.num_layers,
                "num_heads": self.num_heads,
                "intermediate_dim": self.intermediate_dim,
                "patch_size": self.patch_size,
                "image_size": self.image_size,
                "layer_norm_epsilon": self.layer_norm_epsilon,
            }
        )
        return config


class Gemma3VisionAveragePooling(keras.layers.Layer):
    def __init__(self, image_size, patch_size, pool_size, **kwargs):
        super().__init__(**kwargs)

        self.width = image_size // patch_size
        # `reduced_width` is the same as `num_vision_tokens_per_image`.
        self.reduced_width = self.width // pool_size

        # Attributes.
        self.image_size = image_size
        self.patch_size = patch_size
        self.pool_size = pool_size

    def build(self, input_shape):
        self.average_pooling = keras.layers.AveragePooling2D(
            pool_size=self.pool_size,
            strides=self.pool_size,
            padding="valid",
            dtype=self.dtype_policy,
            name="average_pooling",
        )

    def call(self, x):
        # reshape `(bsz, height*width, emb_dim)` to
        # `(bsz, width, width, emb_dim)`. `height` should be equal to
        # `width`.
        batch_size, _, hidden_dim = ops.shape(x)
        x = ops.reshape(x, (batch_size, self.width, self.width, hidden_dim))
        x = self.average_pooling(x)
        output = ops.reshape(
            x, (batch_size, self.reduced_width * self.reduced_width, hidden_dim)
        )
        return output

    def compute_output_shape(self, input_shape):
        return (
            input_shape[0],
            self.reduced_width * self.reduced_width,
            input_shape[-1],
        )

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "image_size": self.image_size,
                "patch_size": self.patch_size,
                "pool_size": self.pool_size,
            }
        )
        return config


class Gemma3VisionOutput(keras.layers.Layer):
    def __init__(
        self,
        output_dim,
        layer_norm_epsilon=1e-6,
        kernel_initializer="glorot_uniform",
        **kwargs,
    ):
        super().__init__(**kwargs)

        self.layer_norm_epsilon = layer_norm_epsilon
        self.output_dim = output_dim

        self._kernel_initializer = keras.initializers.get(
            clone_initializer(kernel_initializer)
        )

    def build(self, input_shape):
        self.vision_soft_embedding_norm = RMSNormalization(
            epsilon=self.layer_norm_epsilon,
            dtype=self.dtype_policy,
            name="vision_soft_embedding_norm",
        )
        self.vision_soft_embedding_norm.build(input_shape)

        self.vision_input_projection = keras.layers.Dense(
            units=self.output_dim,
            use_bias=False,
            kernel_initializer=self._kernel_initializer,
            dtype=self.dtype_policy,
            name="vision_input_projection",
        )
        self.vision_input_projection.build(input_shape)

    def call(self, inputs):
        x = self.vision_soft_embedding_norm(inputs)
        x = self.vision_input_projection(x)
        return x

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "output_dim": self.output_dim,
                "layer_norm_epsilon": self.layer_norm_epsilon,
                "kernel_initializer": keras.initializers.serialize(
                    self._kernel_initializer
                ),
            }
        )

    def compute_output_shape(self, input_shape):
        return input_shape[:-1] + (self.output_dim,)



"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from keras_hub.src.metrics.bleu import Bleu as Bleu
from keras_hub.src.metrics.edit_distance import EditDistance as EditDistance
from keras_hub.src.metrics.perplexity import Perplexity as Perplexity
from keras_hub.src.metrics.rouge_l import RougeL as RougeL
from keras_hub.src.metrics.rouge_n import RougeN as RougeN



from keras_hub.src.models.mixtral.mixtral_backbone import MixtralBackbone
from keras_hub.src.models.mixtral.mixtral_presets import backbone_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, MixtralBackbone)



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.image_classifier import ImageClassifier
from keras_hub.src.models.mit.mit_backbone import MiTBackbone
from keras_hub.src.models.mit.mit_image_classifier_preprocessor import (
    MiTImageClassifierPreprocessor,
)


@keras_hub_export("keras_hub.models.MiTImageClassifier")
class MiTImageClassifier(ImageClassifier):
    backbone_cls = MiTBackbone
    preprocessor_cls = MiTImageClassifierPreprocessor



"""SigLIP model preset configurations."""

# Metadata for loading pretrained model weights.
backbone_presets = {
    "siglip_base_patch16_224": {
        "metadata": {
            "description": (
                "200 million parameter, image size 224, pre-trained on WebLi."
            ),
            "params": 203156230,
            "official_name": "SigLIP",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip-659d5e62f0ae1a57ae0e83ba",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip_base_patch16_224/1",
    },
    "siglip_base_patch16_256": {
        "metadata": {
            "description": (
                "200 million parameter, image size 256, pre-trained on WebLi."
            ),
            "params": 203202370,
            "official_name": "SigLIP",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip-659d5e62f0ae1a57ae0e83ba",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip_base_patch16_256/1",
    },
    "siglip_base_patch16_384": {
        "metadata": {
            "description": (
                "200 million parameter, image size 384, pre-trained on WebLi."
            ),
            "params": 203448450,
            "official_name": "SigLIP",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip-659d5e62f0ae1a57ae0e83ba",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip_base_patch16_384/1",
    },
    "siglip_base_patch16_512": {
        "metadata": {
            "description": (
                "200 million parameter, image size 512, pre-trained on WebLi."
            ),
            "params": 203792962,
            "official_name": "SigLIP",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip-659d5e62f0ae1a57ae0e83ba",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip_base_patch16_512/1",
    },
    "siglip_large_patch16_256": {
        "metadata": {
            "description": (
                "652 million parameter, image size 256, pre-trained on WebLi."
            ),
            "params": 652151106,
            "official_name": "SigLIP",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip-659d5e62f0ae1a57ae0e83ba",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip_large_patch16_256/1",
    },
    "siglip_large_patch16_384": {
        "metadata": {
            "description": (
                "652 million parameter, image size 384, pre-trained on WebLi."
            ),
            "params": 652479106,
            "official_name": "SigLIP",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip-659d5e62f0ae1a57ae0e83ba",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip_large_patch16_384/1",
    },
    "siglip_so400m_patch14_224": {
        "metadata": {
            "description": (
                "877 million parameter, image size 224, "
                "shape-optimized version, pre-trained on WebLi."
            ),
            "params": 877360578,
            "official_name": "SigLIP",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip-659d5e62f0ae1a57ae0e83ba",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip_so400m_patch14_224/1",
    },
    "siglip_so400m_patch14_384": {
        "metadata": {
            "description": (
                "877 million parameter, image size 384, "
                "shape-optimized version, pre-trained on WebLi."
            ),
            "params": 877961291,
            "official_name": "SigLIP",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip-659d5e62f0ae1a57ae0e83ba",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip_so400m_patch14_384/1",
    },
    "siglip_so400m_patch16_256_i18n": {
        "metadata": {
            "description": (
                "1.1 billion parameter, image size 256, "
                "shape-optimized version, pre-trained on WebLi."
            ),
            "params": 1128759282,
            "official_name": "SigLIP",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip-659d5e62f0ae1a57ae0e83ba",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip_so400m_patch16_256_i18n/1",
    },
    "siglip_base_patch16_256_multilingual": {
        "metadata": {
            "description": (
                "370 million parameter, image size 256, pre-trained on WebLi."
            ),
            "params": 370626370,
            "official_name": "SigLIP",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip-659d5e62f0ae1a57ae0e83ba",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip_base_patch16_256_multilingual/1",
    },
    # SigLIP2.
    "siglip2_base_patch16_224": {
        "metadata": {
            "description": (
                "375 million parameter, patch size 16, image size 224, "
                "pre-trained on WebLi."
            ),
            "params": 375188230,
            "official_name": "SigLIP2",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_base_patch16_224/1",
    },
    "siglip2_base_patch16_256": {
        "metadata": {
            "description": (
                "375 million parameter, patch size 16, image size 256, "
                "pre-trained on WebLi."
            ),
            "params": 375234370,
            "official_name": "SigLIP2",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_base_patch16_256/1",
    },
    "siglip2_base_patch32_256": {
        "metadata": {
            "description": (
                "376 million parameter, patch size 32, image size 256, "
                "pre-trained on WebLi."
            ),
            "params": 376856194,
            "official_name": "SigLIP2",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_base_patch32_256/1",
    },
    "siglip2_base_patch16_384": {
        "metadata": {
            "description": (
                "376 million parameter, patch size 16, image size 384, "
                "pre-trained on WebLi."
            ),
            "params": 376856194,
            "official_name": "SigLIP2",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_base_patch16_384/1",
    },
    # "siglip2_base_patch16_512": {
    #     "metadata": {
    #         "description": (
    #             "375 million parameter, patch size 16, image size 512, "
    #             "pre-trained on WebLi."
    #         ),
    #         "params": 375824962,
    #         "official_name": "SigLIP2",
    #         "path": "siglip",
    #         "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
    #     },
    #     "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_base_patch16_512/1",
    # },
    "siglip2_large_patch16_256": {
        "metadata": {
            "description": (
                "881 million parameter, patch size 16, image size 256, "
                "pre-trained on WebLi."
            ),
            "params": 881527106,
            "official_name": "SigLIP2",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_large_patch16_256/1",
    },
    "siglip2_large_patch16_384": {
        "metadata": {
            "description": (
                "881 million parameter, patch size 16, image size 384, "
                "pre-trained on WebLi."
            ),
            "params": 881855106,
            "official_name": "SigLIP2",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_large_patch16_384/1",
    },
    "siglip2_large_patch16_512": {
        "metadata": {
            "description": (
                "882 million parameter, patch size 16, image size 512, "
                "pre-trained on WebLi."
            ),
            "params": 882314306,
            "official_name": "SigLIP2",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_large_patch16_512/1",
    },
    "siglip2_giant_opt_patch16_256": {
        "metadata": {
            "description": (
                "1.8 billion parameter, patch size 16, image size 256, "
                "pre-trained on WebLi."
            ),
            "params": 1871394226,
            "official_name": "SigLIP2",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_giant_opt_patch16_256/2",
    },
    "siglip2_giant_opt_patch16_384": {
        "metadata": {
            "description": (
                "1.8 billion parameter, patch size 16, image size 384, "
                "pre-trained on WebLi."
            ),
            "params": 1871886066,
            "official_name": "SigLIP2",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_giant_opt_patch16_384/1",
    },
    "siglip2_so400m_patch14_224": {
        "metadata": {
            "description": (
                "1.1 billion parameter, patch size 14, image size 224, "
                "shape-optimized version, pre-trained on WebLi."
            ),
            "params": 1135463922,
            "official_name": "SigLIP2",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_so400m_patch14_224/1",
    },
    "siglip2_so400m_patch14_384": {
        "metadata": {
            "description": (
                "1.1 billion parameter, patch size 14, image size 224, "
                "shape-optimized version, pre-trained on WebLi."
            ),
            "params": 1136009291,
            "official_name": "SigLIP2",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_so400m_patch14_384/1",
    },
    "siglip2_so400m_patch16_256": {
        "metadata": {
            "description": (
                "1.1 billion parameter, patch size 16, image size 256, "
                "shape-optimized version, pre-trained on WebLi."
            ),
            "params": 1135671282,
            "official_name": "SigLIP2",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_so400m_patch16_256/1",
    },
    "siglip2_so400m_patch16_384": {
        "metadata": {
            "description": (
                "1.1 billion parameter, patch size 16, image size 384, "
                "shape-optimized version, pre-trained on WebLi."
            ),
            "params": 1136040242,
            "official_name": "SigLIP2",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_so400m_patch16_384/1",
    },
    "siglip2_so400m_patch16_512": {
        "metadata": {
            "description": (
                "1.1 billion parameter, patch size 16, image size 512, "
                "shape-optimized version, pre-trained on WebLi."
            ),
            "params": 1136555698,
            "official_name": "SigLIP2",
            "path": "siglip",
            "model_card": "https://huggingface.co/collections/google/siglip2-67b5dcef38c175486e240107",
        },
        "kaggle_handle": "kaggle://keras/siglip/keras/siglip2_so400m_patch16_512/1",
    },
}



"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from keras_hub.src.layers.modeling.alibi_bias import AlibiBias as AlibiBias
from keras_hub.src.layers.modeling.anchor_generator import (
    AnchorGenerator as AnchorGenerator,
)
from keras_hub.src.layers.modeling.box_matcher import BoxMatcher as BoxMatcher
from keras_hub.src.layers.modeling.cached_multi_head_attention import (
    CachedMultiHeadAttention as CachedMultiHeadAttention,
)
from keras_hub.src.layers.modeling.f_net_encoder import (
    FNetEncoder as FNetEncoder,
)
from keras_hub.src.layers.modeling.masked_lm_head import (
    MaskedLMHead as MaskedLMHead,
)
from keras_hub.src.layers.modeling.non_max_supression import (
    NonMaxSuppression as NonMaxSuppression,
)
from keras_hub.src.layers.modeling.position_embedding import (
    PositionEmbedding as PositionEmbedding,
)
from keras_hub.src.layers.modeling.reversible_embedding import (
    ReversibleEmbedding as ReversibleEmbedding,
)
from keras_hub.src.layers.modeling.rms_normalization import (
    RMSNormalization as RMSNormalization,
)
from keras_hub.src.layers.modeling.rotary_embedding import (
    RotaryEmbedding as RotaryEmbedding,
)
from keras_hub.src.layers.modeling.sine_position_encoding import (
    SinePositionEncoding as SinePositionEncoding,
)
from keras_hub.src.layers.modeling.token_and_position_embedding import (
    TokenAndPositionEmbedding as TokenAndPositionEmbedding,
)
from keras_hub.src.layers.modeling.transformer_decoder import (
    TransformerDecoder as TransformerDecoder,
)
from keras_hub.src.layers.modeling.transformer_encoder import (
    TransformerEncoder as TransformerEncoder,
)
from keras_hub.src.layers.preprocessing.audio_converter import (
    AudioConverter as AudioConverter,
)
from keras_hub.src.layers.preprocessing.image_converter import (
    ImageConverter as ImageConverter,
)
from keras_hub.src.layers.preprocessing.masked_lm_mask_generator import (
    MaskedLMMaskGenerator as MaskedLMMaskGenerator,
)
from keras_hub.src.layers.preprocessing.multi_segment_packer import (
    MultiSegmentPacker as MultiSegmentPacker,
)
from keras_hub.src.layers.preprocessing.random_deletion import (
    RandomDeletion as RandomDeletion,
)
from keras_hub.src.layers.preprocessing.random_swap import (
    RandomSwap as RandomSwap,
)
from keras_hub.src.layers.preprocessing.start_end_packer import (
    StartEndPacker as StartEndPacker,
)
from keras_hub.src.models.basnet.basnet_image_converter import (
    BASNetImageConverter as BASNetImageConverter,
)
from keras_hub.src.models.clip.clip_image_converter import (
    CLIPImageConverter as CLIPImageConverter,
)
from keras_hub.src.models.cspnet.cspnet_image_converter import (
    CSPNetImageConverter as CSPNetImageConverter,
)
from keras_hub.src.models.deeplab_v3.deeplab_v3_image_converter import (
    DeepLabV3ImageConverter as DeepLabV3ImageConverter,
)
from keras_hub.src.models.densenet.densenet_image_converter import (
    DenseNetImageConverter as DenseNetImageConverter,
)
from keras_hub.src.models.efficientnet.efficientnet_image_converter import (
    EfficientNetImageConverter as EfficientNetImageConverter,
)
from keras_hub.src.models.gemma3.gemma3_image_converter import (
    Gemma3ImageConverter as Gemma3ImageConverter,
)
from keras_hub.src.models.mit.mit_image_converter import (
    MiTImageConverter as MiTImageConverter,
)
from keras_hub.src.models.mobilenet.mobilenet_image_converter import (
    MobileNetImageConverter as MobileNetImageConverter,
)
from keras_hub.src.models.moonshine.moonshine_audio_converter import (
    MoonshineAudioConverter as MoonshineAudioConverter,
)
from keras_hub.src.models.pali_gemma.pali_gemma_image_converter import (
    PaliGemmaImageConverter as PaliGemmaImageConverter,
)
from keras_hub.src.models.resnet.resnet_image_converter import (
    ResNetImageConverter as ResNetImageConverter,
)
from keras_hub.src.models.retinanet.retinanet_image_converter import (
    RetinaNetImageConverter as RetinaNetImageConverter,
)
from keras_hub.src.models.sam.sam_image_converter import (
    SAMImageConverter as SAMImageConverter,
)
from keras_hub.src.models.sam.sam_mask_decoder import (
    SAMMaskDecoder as SAMMaskDecoder,
)
from keras_hub.src.models.sam.sam_prompt_encoder import (
    SAMPromptEncoder as SAMPromptEncoder,
)
from keras_hub.src.models.segformer.segformer_image_converter import (
    SegFormerImageConverter as SegFormerImageConverter,
)
from keras_hub.src.models.siglip.siglip_image_converter import (
    SigLIPImageConverter as SigLIPImageConverter,
)
from keras_hub.src.models.vgg.vgg_image_converter import (
    VGGImageConverter as VGGImageConverter,
)
from keras_hub.src.models.vit.vit_image_converter import (
    ViTImageConverter as ViTImageConverter,
)
from keras_hub.src.models.whisper.whisper_audio_converter import (
    WhisperAudioConverter as WhisperAudioConverter,
)
from keras_hub.src.models.xception.xception_image_converter import (
    XceptionImageConverter as XceptionImageConverter,
)



from keras_hub.src.models.siglip.siglip_backbone import SigLIPBackbone
from keras_hub.src.models.siglip.siglip_presets import backbone_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, SigLIPBackbone)



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.image_classifier import ImageClassifier
from keras_hub.src.models.resnet.resnet_backbone import ResNetBackbone
from keras_hub.src.models.resnet.resnet_image_classifier_preprocessor import (
    ResNetImageClassifierPreprocessor,
)


@keras_hub_export("keras_hub.models.ResNetImageClassifier")
class ResNetImageClassifier(ImageClassifier):
    backbone_cls = ResNetBackbone
    preprocessor_cls = ResNetImageClassifierPreprocessor



"""CLIP model preset configurations."""

# Metadata for loading pretrained model weights.
backbone_presets = {
    "clip_vit_base_patch16": {
        "metadata": {
            "description": (
                "150 million parameter, 12-layer for vision and 12-layer for "
                "text, patch size of 16, CLIP model."
            ),
            "params": 149620934,
            "path": "clip",
        },
        "kaggle_handle": "kaggle://keras/clip/keras/clip_vit_base_patch16/2",
    },
    "clip_vit_base_patch32": {
        "metadata": {
            "description": (
                "151 million parameter, 12-layer for vision and 12-layer for "
                "text, patch size of 32, CLIP model."
            ),
            "params": 151277363,
            "path": "clip",
        },
        "kaggle_handle": "kaggle://keras/clip/keras/clip_vit_base_patch32/2",
    },
    "clip_vit_large_patch14": {
        "metadata": {
            "description": (
                "428 million parameter, 24-layer for vision and 12-layer for "
                "text, patch size of 14, CLIP model."
            ),
            "params": 427616770,
            "path": "clip",
        },
        "kaggle_handle": "kaggle://keras/clip/keras/clip_vit_large_patch14/2",
    },
    "clip_vit_large_patch14_336": {
        "metadata": {
            "description": (
                "428 million parameter, 24-layer for vision and 12-layer for "
                "text, patch size of 14, image size of 336, CLIP model."
            ),
            "params": 427944770,
            "path": "clip",
        },
        "kaggle_handle": "kaggle://keras/clip/keras/clip_vit_large_patch14_336/2",
    },
    "clip_vit_b_32_laion2b_s34b_b79k": {
        "metadata": {
            "description": (
                "151 million parameter, 12-layer for vision and 12-layer for "
                "text, patch size of 32, Open CLIP model."
            ),
            "params": 151277363,
            "path": "clip",
        },
        "kaggle_handle": "kaggle://keras/clip/keras/clip_vit_b_32_laion2b_s34b_b79k/2",
    },
    "clip_vit_h_14_laion2b_s32b_b79k": {
        "metadata": {
            "description": (
                "986 million parameter, 32-layer for vision and 24-layer for "
                "text, patch size of 14, Open CLIP model."
            ),
            "params": 986109698,
            "path": "clip",
        },
        "kaggle_handle": "kaggle://keras/clip/keras/clip_vit_h_14_laion2b_s32b_b79k/2",
    },
    "clip_vit_g_14_laion2b_s12b_b42k": {
        "metadata": {
            "description": (
                "1.4 billion parameter, 40-layer for vision and 24-layer for "
                "text, patch size of 14, Open CLIP model."
            ),
            "params": 1366678530,
            "path": "clip",
        },
        "kaggle_handle": "kaggle://keras/clip/keras/clip_vit_g_14_laion2b_s12b_b42k/2",
    },
    "clip_vit_bigg_14_laion2b_39b_b160k": {
        "metadata": {
            "description": (
                "2.5 billion parameter, 48-layer for vision and 32-layer for "
                "text, patch size of 14, Open CLIP model."
            ),
            "params": 2539567362,
            "path": "clip",
        },
        "kaggle_handle": "kaggle://keras/clip/keras/clip_vit_bigg_14_laion2b_39b_b160k/2",
    },
}



from keras_hub.src.models.gemma.gemma_backbone import GemmaBackbone
from keras_hub.src.models.gemma.gemma_presets import backbone_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, GemmaBackbone)



"""Convert huggingface models to KerasHub."""

from keras_hub.src.models.image_classifier import ImageClassifier
from keras_hub.src.utils.preset_utils import PresetLoader
from keras_hub.src.utils.preset_utils import jax_memory_cleanup
from keras_hub.src.utils.transformers import convert_albert
from keras_hub.src.utils.transformers import convert_bart
from keras_hub.src.utils.transformers import convert_bert
from keras_hub.src.utils.transformers import convert_distilbert
from keras_hub.src.utils.transformers import convert_gemma
from keras_hub.src.utils.transformers import convert_gpt2
from keras_hub.src.utils.transformers import convert_llama3
from keras_hub.src.utils.transformers import convert_mistral
from keras_hub.src.utils.transformers import convert_mixtral
from keras_hub.src.utils.transformers import convert_pali_gemma
from keras_hub.src.utils.transformers import convert_qwen
from keras_hub.src.utils.transformers import convert_qwen_moe
from keras_hub.src.utils.transformers import convert_vit
from keras_hub.src.utils.transformers.safetensor_utils import SafetensorLoader


class TransformersPresetLoader(PresetLoader):
    def __init__(self, preset, config):
        super().__init__(preset, config)
        model_type = self.config["model_type"]
        if model_type == "albert":
            self.converter = convert_albert
        elif model_type == "bart":
            self.converter = convert_bart
        elif model_type == "bert":
            self.converter = convert_bert
        elif model_type == "distilbert":
            self.converter = convert_distilbert
        elif model_type == "gemma" or model_type == "gemma2":
            self.converter = convert_gemma
        elif model_type == "gpt2":
            self.converter = convert_gpt2
        elif model_type == "llama":
            # TODO: handle other llama versions.
            self.converter = convert_llama3
        elif model_type == "mistral":
            self.converter = convert_mistral
        elif model_type == "paligemma":
            self.converter = convert_pali_gemma
        elif model_type == "vit":
            self.converter = convert_vit
        elif model_type == "qwen2":
            self.converter = convert_qwen
        elif model_type == "mixtral":
            self.converter = convert_mixtral
        elif model_type == "qwen2_moe":
            self.converter = convert_qwen_moe
        else:
            raise ValueError(
                "KerasHub has no converter for huggingface/transformers models "
                f"with model type `'{model_type}'`."
            )

    def check_backbone_class(self):
        return self.converter.backbone_cls

    def load_backbone(self, cls, load_weights, **kwargs):
        keras_config = self.converter.convert_backbone_config(self.config)
        backbone = cls(**{**keras_config, **kwargs})
        if load_weights:
            jax_memory_cleanup(backbone)
            with SafetensorLoader(self.preset) as loader:
                self.converter.convert_weights(backbone, loader, self.config)
        return backbone

    def load_task(self, cls, load_weights, load_task_weights, **kwargs):
        architecture = self.config["architectures"][0]
        if (
            not load_task_weights
            or not issubclass(cls, ImageClassifier)
            or architecture == "ViTModel"
        ):
            return super().load_task(
                cls, load_weights, load_task_weights, **kwargs
            )
        # Support loading the classification head for classifier models.
        if architecture == "ViTForImageClassification":
            kwargs["num_classes"] = len(self.config["id2label"])
        task = super().load_task(cls, load_weights, load_task_weights, **kwargs)
        if load_task_weights:
            with SafetensorLoader(self.preset, prefix="") as loader:
                self.converter.convert_head(task, loader, self.config)
        return task

    def load_tokenizer(self, cls, config_name="tokenizer.json", **kwargs):
        return self.converter.convert_tokenizer(cls, self.preset, **kwargs)

    def load_image_converter(self, cls, **kwargs):
        # TODO: set image size for pali gemma checkpoints.
        return None



import numpy as np

from keras_hub.src.models.mistral.mistral_backbone import MistralBackbone
from keras_hub.src.utils.preset_utils import get_file

backbone_cls = MistralBackbone


def convert_backbone_config(transformers_config):
    return {
        "vocabulary_size": transformers_config["vocab_size"],
        "num_layers": transformers_config["num_hidden_layers"],
        "num_query_heads": transformers_config["num_attention_heads"],
        "hidden_dim": transformers_config["hidden_size"],
        "intermediate_dim": transformers_config["intermediate_size"],
        "num_key_value_heads": transformers_config["num_key_value_heads"],
        "rope_max_wavelength": transformers_config["rope_theta"],
        "layer_norm_epsilon": transformers_config["rms_norm_eps"],
        "sliding_window": transformers_config["sliding_window"],
    }


def convert_weights(backbone, loader, transformers_config):
    # Embeddings
    loader.port_weight(
        keras_variable=backbone.token_embedding.embeddings,
        hf_weight_key="model.embed_tokens.weight",
        hook_fn=lambda hf_tensor, _: hf_tensor.astype(np.float16),
    )
    loader.port_weight(
        keras_variable=backbone.token_embedding.reverse_embeddings,
        hf_weight_key="lm_head.weight",
        hook_fn=lambda hf_tensor, _: np.transpose(
            hf_tensor.astype(np.float16), axes=(1, 0)
        ),
    )

    # Attention blocks
    for index in range(backbone.num_layers):
        decoder_layer = backbone.transformer_layers[index]

        # Norm layers
        loader.port_weight(
            keras_variable=decoder_layer._self_attention_layernorm.scale,
            hf_weight_key=f"model.layers.{index}.input_layernorm.weight",
            hook_fn=lambda hf_tensor, _: hf_tensor.astype(np.float16),
        )
        loader.port_weight(
            keras_variable=decoder_layer._feedforward_layernorm.scale,
            hf_weight_key=f"model.layers.{index}.post_attention_layernorm.weight",
            hook_fn=lambda hf_tensor, _: hf_tensor.astype(np.float16),
        )

        # Attention layers
        loader.port_weight(
            keras_variable=decoder_layer._self_attention_layer._query_dense.kernel,
            hf_weight_key=f"model.layers.{index}.self_attn.q_proj.weight",
            hook_fn=lambda hf_tensor, keras_shape: np.reshape(
                np.transpose(hf_tensor.astype(np.float16)), keras_shape
            ),
        )
        loader.port_weight(
            keras_variable=decoder_layer._self_attention_layer._key_dense.kernel,
            hf_weight_key=f"model.layers.{index}.self_attn.k_proj.weight",
            hook_fn=lambda hf_tensor, keras_shape: np.reshape(
                np.transpose(hf_tensor.astype(np.float16)), keras_shape
            ),
        )
        loader.port_weight(
            keras_variable=decoder_layer._self_attention_layer._value_dense.kernel,
            hf_weight_key=f"model.layers.{index}.self_attn.v_proj.weight",
            hook_fn=lambda hf_tensor, keras_shape: np.reshape(
                np.transpose(hf_tensor.astype(np.float16)), keras_shape
            ),
        )
        loader.port_weight(
            keras_variable=decoder_layer._self_attention_layer._output_dense.kernel,
            hf_weight_key=f"model.layers.{index}.self_attn.o_proj.weight",
            hook_fn=lambda hf_tensor, keras_shape: np.reshape(
                np.transpose(hf_tensor.astype(np.float16)), keras_shape
            ),
        )

        # MLP layers
        loader.port_weight(
            keras_variable=decoder_layer._feedforward_gate_dense.kernel,
            hf_weight_key=f"model.layers.{index}.mlp.gate_proj.weight",
            hook_fn=lambda hf_tensor, _: np.transpose(
                hf_tensor.astype(np.float16), axes=(1, 0)
            ),
        )
        loader.port_weight(
            keras_variable=decoder_layer._feedforward_intermediate_dense.kernel,
            hf_weight_key=f"model.layers.{index}.mlp.up_proj.weight",
            hook_fn=lambda hf_tensor, _: np.transpose(
                hf_tensor.astype(np.float16), axes=(1, 0)
            ),
        )
        loader.port_weight(
            keras_variable=decoder_layer._feedforward_output_dense.kernel,
            hf_weight_key=f"model.layers.{index}.mlp.down_proj.weight",
            hook_fn=lambda hf_tensor, _: np.transpose(
                hf_tensor.astype(np.float16), axes=(1, 0)
            ),
        )

    # Normalization
    loader.port_weight(
        keras_variable=backbone.layer_norm.scale,
        hf_weight_key="model.norm.weight",
        hook_fn=lambda hf_tensor, _: hf_tensor.astype(np.float16),
    )


def convert_tokenizer(cls, preset, **kwargs):
    return cls(get_file(preset, "tokenizer.model"), **kwargs)



import keras
from keras import layers

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.flux.flux_model import FluxBackbone
from keras_hub.src.models.preprocessor import Preprocessor


@keras_hub_export("keras_hub.models.FluxTextToImagePreprocessor")
class FluxTextToImagePreprocessor(Preprocessor):
    """Flux text-to-image model preprocessor.

    This preprocessing layer is meant for use with
    `keras_hub.models.FluxTextToImagePreprocessor`.

    For use with generation, the layer exposes one methods
    `generate_preprocess()`.

    Args:
        clip_l_preprocessor: A `keras_hub.models.CLIPPreprocessor` instance.
        t5_preprocessor: A optional `keras_hub.models.T5Preprocessor` instance.
    """

    backbone_cls = FluxBackbone

    def __init__(
        self,
        clip_l_preprocessor,
        t5_preprocessor=None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.clip_l_preprocessor = clip_l_preprocessor
        self.t5_preprocessor = t5_preprocessor

    @property
    def sequence_length(self):
        """The padded length of model input sequences."""
        return self.clip_l_preprocessor.sequence_length

    def build(self, input_shape):
        self.built = True

    def generate_preprocess(self, x):
        token_ids = {}
        token_ids["clip_l"] = self.clip_l_preprocessor(x)["token_ids"]
        if self.t5_preprocessor is not None:
            token_ids["t5"] = self.t5_preprocessor(x)["token_ids"]
        return token_ids

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "clip_l_preprocessor": layers.serialize(
                    self.clip_l_preprocessor
                ),
                "t5_preprocessor": layers.serialize(self.t5_preprocessor),
            }
        )
        return config

    @classmethod
    def from_config(cls, config):
        for layer_name in (
            "clip_l_preprocessor",
            "t5_preprocessor",
        ):
            if layer_name in config and isinstance(config[layer_name], dict):
                config[layer_name] = keras.layers.deserialize(
                    config[layer_name]
                )
        return cls(**config)



def adjust_channels(x, divisor=8, min_value=None):
    """Ensure that all layers have a channel number divisible by the `divisor`.

    Args:
        x: integer, input value.
        divisor: integer, the value by which a channel number should be
            divisible, defaults to 8.
        min_value: float, optional minimum value for the new tensor. If None,
            defaults to value of divisor.

    Returns:
        the updated input scalar.
    """

    if min_value is None:
        min_value = divisor

    new_x = max(min_value, int(x + divisor / 2) // divisor * divisor)

    # make sure that round down does not go down by more than 10%.
    if new_x < 0.9 * x:
        new_x += divisor
    return new_x



"""SegFormer model preset configurations."""

presets = {
    "segformer_b0_ade20k_512": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB0 backbone fine-tuned on ADE20k in "
                "512x512 resolution."
            ),
            "params": 3719027,
            "path": "segformer_b0",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b0_ade20k_512/3",
    },
    "segformer_b1_ade20k_512": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB1 backbone fine-tuned on ADE20k in "
                "512x512 resolution."
            ),
            "params": 13682643,
            "path": "segformer_b1",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b1_ade20k_512/5",
    },
    "segformer_b2_ade20k_512": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB2 backbone fine-tuned on ADE20k in "
                "512x512 resolution."
            ),
            "params": 24727507,
            "path": "segformer_b2",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b2_ade20k_512/3",
    },
    "segformer_b3_ade20k_512": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB3 backbone fine-tuned on ADE20k in "
                "512x512 resolution."
            ),
            "params": 44603347,
            "path": "segformer_b3",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b3_ade20k_512/3",
    },
    "segformer_b4_ade20k_512": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB4 backbone fine-tuned on ADE20k in "
                "512x512 resolution."
            ),
            "params": 61373907,
            "path": "segformer_b4",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b4_ade20k_512/3",
    },
    "segformer_b5_ade20k_640": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB5 backbone fine-tuned on ADE20k in "
                "640x640 resolution."
            ),
            "params": 81974227,
            "path": "segformer_b5",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b5_ade20k_640/3",
    },
    "segformer_b0_cityscapes_1024": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB0 backbone fine-tuned on Cityscapes "
                "in 1024x1024 resolution."
            ),
            "params": 3719027,
            "path": "segformer_b0",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b0_cityscapes_1024/3",
    },
    "segformer_b1_cityscapes_1024": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB1 backbone fine-tuned on Cityscapes "
                "in 1024x1024 resolution."
            ),
            "params": 13682643,
            "path": "segformer_b1",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b1_ade20k_512/1",
    },
    "segformer_b2_cityscapes_1024": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB2 backbone fine-tuned on Cityscapes "
                "in 1024x1024 resolution."
            ),
            "params": 24727507,
            "path": "segformer_b2",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b2_cityscapes_1024/3",
    },
    "segformer_b3_cityscapes_1024": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB3 backbone fine-tuned on Cityscapes "
                "in 1024x1024 resolution."
            ),
            "params": 44603347,
            "path": "segformer_b3",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b3_cityscapes_1024/3",
    },
    "segformer_b4_cityscapes_1024": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB4 backbone fine-tuned on Cityscapes "
                "in 1024x1024 resolution."
            ),
            "params": 61373907,
            "path": "segformer_b4",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b4_cityscapes_1024/3",
    },
    "segformer_b5_cityscapes_1024": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB5 backbone fine-tuned on Cityscapes "
                "in 1024x1024 resolution."
            ),
            "params": 81974227,
            "path": "segformer_b5",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b5_cityscapes_1024/3",
    },
}



import keras
from keras import ops

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.modeling.reversible_embedding import (
    ReversibleEmbedding,
)
from keras_hub.src.models.backbone import Backbone
from keras_hub.src.models.gemma.rms_normalization import RMSNormalization
from keras_hub.src.models.gemma3.gemma3_decoder_block import Gemma3DecoderBlock
from keras_hub.src.models.gemma3.gemma3_interleave_embeddings import (
    Gemma3InterleaveEmbeddings,
)


@keras_hub_export("keras_hub.models.Gemma3Backbone")
class Gemma3Backbone(Backbone):
    """Gemma3 core network with hyperparameters.

    This backbone implements the Gemma3 model architecture. Gemma3 is a
    vision-language model (image-text in, text out). The text input is encoded
    using an embedding layer; images are encoded using a vision transformer
    (ViT). After encoding these two modalities, the image embeddings are placed
    in the correct position in the text embedding sequence. The mixed sequence
    of embeddings is then passed through transformer decoder layers.

    For a higher-level object for text-generation, see
    `keras_hub.models.Gemma3CausalLM`.

    The default constructor gives a fully customizable, randomly initialized
    Gemma3 model with any vision encoder, number of heads, embedding dimensions,
    and equivalent configuration for the decoder layers. To load preset
    architectures and weights, use the `from_preset` constructor.

    Args:
        vocabulary_size: int. The size of the token vocabulary.
        image_size: int. The resolution of the image in both width and height.
            The input images must be square.
        num_layers: int. The number of transformer mixed decoder layers.
        num_query_heads: int. The number of heads for the query projections in
            the mixed decoder attention layer.
        num_key_value_heads: int. The number of heads for the key and value
            projections in the mixed decoder attention layers.
        hidden_dim: int. The size of the transformer hidden state at the end
            of each mixed transformer layer.
        intermediate_dim: int. The output dimension of the first Dense layer in
            a two-layer feedforward network for each transformer decoder block.
        head_dim: int. The size of each attention head in the mixed decoder.
        query_head_dim_normalize: boolean. If `True` normalize the query before
            attention with `head_dim`. If `False`, normalize the query with
            `hidden_dim / num_query_heads`. Defaults to `True`.
        use_query_key_norm: bool. If `True`, apply a RMS Norm layer to query and
            key before projecting them. Defaults to `True`.
        use_post_ffw_norm: boolean. Whether to normalize after the feedforward
            block. Defaults to `False`.
        use_post_attention_norm: boolean. Whether to normalize after the
            attention block. Defaults to `False`.
        attention_logit_soft_cap: `None` or int. Soft cap for the attention
            logits. Defaults to `None`.
        final_logit_soft_cap: `None` or int. Soft cap for the final logits.
            Defaults to `None`.
        use_sliding_window_attention: boolean. Whether to use sliding local
          window attention. Defaults to `False`.
        sliding_window_size: int. Size of the sliding local window. Defaults to
            `4096`.
        vision_encoder: A `Gemma3VisionEncoder` instance. `call()`
            takes in images and returns corresponding sequence of embeddings. If
            `None`, the model is a text-only model.
        layer_norm_epsilon: float. The epsilon value user for every layer norm
            in all transformer blocks. Defaults to `1e-6`.
        dropout: float. Dropout probability for the Transformer decoder blocks.
            Defaults to `0`.
        dtype: string or `keras.mixed_precision.DTypePolicy`. The dtype to use
            for the models computations and weights. Note that some
            computations, such as softmax and layer normalization will always
            be done in float32 precision regardless of dtype. Defaults to
            `bfloat16`.

    Example:
    ```python
    # === Language Gemma3 model ===
    input_data = {}
    input_data["token_ids"] = np.ones(shape=(1, 300), dtype="int32")
    input_data["padding_mask"] = (
        np.expand_dims(np.array([1] * 280 + [0] * (300 - 280)), axis=0)
        .astype(bool)
    )

    # Pretrained Gemma3 decoder.
    model = keras_hub.models.Gemma3Backbone.from_preset(
        "gemma3_instruct_4b_text"
    )
    model(input_data)

    # Randomly initialized Gemma3 decoder with a custom config.
    model = keras_hub.models.Gemma3Backbone(
        vocabulary_size=262144,
        image_size=896,
        num_layers=34,
        num_query_heads=8,
        num_key_value_heads=4,
        hidden_dim=2560,
        intermediate_dim=10240,
        head_dim=256,
        query_head_dim_normalize=True,
        use_post_ffw_norm=True,
        use_post_attention_norm=True,
        final_logit_soft_cap=None,
        attention_logit_soft_cap=None,
        sliding_window_size=1024,
        use_sliding_window_attention=True,
        vision_encoder=None,
        layer_norm_epsilon=1e-06,
        dtype="bfloat16",
    )
    model(input_data)

    # === Vision + Language Gemma3 model ===
    input_data = {}
    input_data["images"] = np.ones(shape=(1, 1, 896, 896, 3))
    input_data["token_ids"] = np.ones(shape=(1, 300), dtype="int32")
    # images after the text part of the sequence.
    input_data["vision_mask"] = np.expand_dims(
        np.array([0] * 30 + [1] * 256 + [0] * 14),
        axis=0,
    ).astype(bool)
    input_data["vision_indices"] = (
        np.expand_dims(np.arange(30, 286), axis=0)
    )
    input_data["padding_mask"] = (
        np.expand_dims(np.array([1] * 286 + [0] * (300 - 286)), axis=0)
        .astype(bool)
    )

    # Pretrained Gemma3 decoder.
    model = keras_hub.models.Gemma3Backbone.from_preset("gemma3_instruct_4b")
    model(input_data)

    # Randomly initialized Gemma3 decoder with a custom config.
    vision_encoder = Gemma3VisionEncoder(
        image_size=896,
        patch_size=14,
        num_heads=16,
        hidden_dim=1152,
        num_layers=27,
        intermediate_dim=4304,
        output_dim=2560,
        pool_size=4,
        layer_norm_epsilon=1e-6,
        dtype="float32",
    )

    model = keras_hub.models.Gemma3Backbone(
        vocabulary_size=262144,
        image_size=896,
        num_layers=34,
        num_query_heads=8,
        num_key_value_heads=4,
        hidden_dim=2560,
        intermediate_dim=10240,
        head_dim=256,
        query_head_dim_normalize=True,
        use_post_ffw_norm=True,
        use_post_attention_norm=True,
        final_logit_soft_cap=None,
        attention_logit_soft_cap=None,
        sliding_window_size=1024,
        use_sliding_window_attention=True,
        vision_encoder=vision_encoder,
        layer_norm_epsilon=1e-06,
        dtype="bfloat16"
    )
    model(input_data)
    ```
    """

    def __init__(
        self,
        vocabulary_size,
        image_size,
        num_layers,
        num_query_heads,
        num_key_value_heads,
        hidden_dim,
        intermediate_dim,
        head_dim,
        query_head_dim_normalize=True,
        use_query_key_norm=True,
        use_post_ffw_norm=False,
        use_post_attention_norm=False,
        attention_logit_soft_cap=None,
        final_logit_soft_cap=None,
        use_sliding_window_attention=False,
        sliding_window_size=1024,
        local_rope_scaling_factor=1.0,
        global_rope_scaling_factor=1.0,
        vision_encoder=None,
        layer_norm_epsilon=1e-6,
        dropout=0,
        dtype=None,
        **kwargs,
    ):
        # === Layers ===
        self.token_embedding = ReversibleEmbedding(
            input_dim=vocabulary_size,
            output_dim=hidden_dim,
            tie_weights=True,
            embeddings_initializer=keras.initializers.VarianceScaling(
                scale=1.0,
                mode="fan_in",
                distribution="untruncated_normal",
                seed=None,
            ),
            dtype=dtype,
            logit_soft_cap=final_logit_soft_cap,
            name="token_embedding",
        )

        self.vision_encoder = vision_encoder
        text_only_model = True if vision_encoder is None else False
        if not text_only_model:
            self.interleave_embeddings = Gemma3InterleaveEmbeddings(
                num_vision_tokens_per_image=self.vision_encoder.num_vision_tokens_per_image,
                dtype=dtype,
                name="interleave_embeddings",
            )

        self.transformer_layers = []
        for i in range(num_layers):
            # 5 local, 1 global
            sliding_window = use_sliding_window_attention and (i % 6 < 5)
            rope_wavelength = 10_000.0 if sliding_window else 1_000_000.0
            rope_scaling_factor = (
                local_rope_scaling_factor
                if sliding_window
                else global_rope_scaling_factor
            )
            layer = Gemma3DecoderBlock(
                hidden_dim=hidden_dim,
                intermediate_dim=intermediate_dim,
                head_dim=head_dim,
                num_query_heads=num_query_heads,
                num_key_value_heads=num_key_value_heads,
                query_head_dim_normalize=query_head_dim_normalize,
                use_query_key_norm=use_query_key_norm,
                use_post_ffw_norm=use_post_ffw_norm,
                use_post_attention_norm=use_post_attention_norm,
                gate_dim_reduction=1,
                logit_soft_cap=attention_logit_soft_cap,
                use_sliding_window_attention=sliding_window,
                sliding_window_size=sliding_window_size,
                rope_wavelength=rope_wavelength,
                rope_scaling_factor=rope_scaling_factor,
                dropout=dropout,
                dtype=dtype,
                name=f"decoder_block_{i}",
            )
            self.transformer_layers.append(layer)
        self.layer_norm = RMSNormalization(
            epsilon=layer_norm_epsilon,
            dtype=dtype,
            name="final_normalization",
        )

        # === Functional Model ===

        # == Model inputs ==
        if not text_only_model:
            image_input = keras.Input(
                shape=(None, image_size, image_size, 3),
                name="images",
            )
            vision_indices_input = keras.Input(
                shape=(None,), dtype="int32", name="vision_indices"
            )
            # Truth be told, this is redundant, and we can infer this from
            # `vision_indices_input`. But it is easier to return this from
            # the preprocessor than to compute it here.
            vision_mask_input = keras.Input(
                shape=(None,), dtype="int32", name="vision_mask"
            )

        token_id_input = keras.Input(
            shape=(None,), dtype="int32", name="token_ids"
        )
        padding_mask_input = keras.Input(
            shape=(None,), dtype="int32", name="padding_mask"
        )

        # == Text embeddings ==
        text_embeddings = self.token_embedding(token_id_input)

        text_embeddings = text_embeddings * ops.cast(
            ops.sqrt(hidden_dim), text_embeddings.dtype
        )

        # == Image Embeddings ==
        if not text_only_model:
            img_embeddings = self.vision_encoder(image_input)

            # == Interleaving text and images ==
            # Place image embeddings in the right position in
            # `text_embeddings`.
            x = self.interleave_embeddings(
                image_embeddings=img_embeddings,
                text_embeddings=text_embeddings,
                vision_indices=vision_indices_input,
            )
        else:
            x = text_embeddings

        # == Decoder layers ==
        for transformer_layer in self.transformer_layers:
            x = transformer_layer(
                x,
                padding_mask=padding_mask_input,
                vision_mask=None if text_only_model else vision_mask_input,
            )
        sequence_output = self.layer_norm(x)

        inputs = {
            "token_ids": token_id_input,
            "padding_mask": padding_mask_input,
        }
        if not text_only_model:
            inputs.update(
                {
                    "images": image_input,
                    "vision_indices": vision_indices_input,
                    "vision_mask": vision_mask_input,
                }
            )

        super().__init__(
            inputs=inputs,
            outputs=sequence_output,
            dtype=dtype,
            **kwargs,
        )

        # === Config ===
        self.vocabulary_size = vocabulary_size
        self.image_size = image_size
        self.num_layers = num_layers
        self.num_query_heads = num_query_heads
        self.num_key_value_heads = num_key_value_heads
        self.hidden_dim = hidden_dim
        self.intermediate_dim = intermediate_dim
        self.head_dim = head_dim
        self.query_head_dim_normalize = query_head_dim_normalize
        self.use_query_key_norm = use_query_key_norm
        self.use_post_ffw_norm = use_post_ffw_norm
        self.use_post_attention_norm = use_post_attention_norm
        self.attention_logit_soft_cap = attention_logit_soft_cap
        self.final_logit_soft_cap = final_logit_soft_cap
        self.use_sliding_window_attention = use_sliding_window_attention
        self.sliding_window_size = sliding_window_size
        self.local_rope_scaling_factor = local_rope_scaling_factor
        self.global_rope_scaling_factor = global_rope_scaling_factor
        self.layer_norm_epsilon = layer_norm_epsilon
        self.dropout = dropout

        # Keep `num_vision_tokens_per_image` as a backbone property for easy
        # access.
        if not text_only_model:
            self.num_vision_tokens_per_image = (
                self.vision_encoder.num_vision_tokens_per_image
            )
        # Also, the `text_only_model`.
        self.text_only_model = text_only_model

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "vocabulary_size": self.vocabulary_size,
                "image_size": self.image_size,
                "num_layers": self.num_layers,
                "num_query_heads": self.num_query_heads,
                "num_key_value_heads": self.num_key_value_heads,
                "hidden_dim": self.hidden_dim,
                "intermediate_dim": self.intermediate_dim,
                "head_dim": self.head_dim,
                "query_head_dim_normalize": self.query_head_dim_normalize,
                "use_query_key_norm": self.use_query_key_norm,
                "use_post_ffw_norm": self.use_post_ffw_norm,
                "use_post_attention_norm": self.use_post_attention_norm,
                "attention_logit_soft_cap": self.attention_logit_soft_cap,
                "final_logit_soft_cap": self.final_logit_soft_cap,
                "use_sliding_window_attention": (
                    self.use_sliding_window_attention
                ),
                "sliding_window_size": self.sliding_window_size,
                "local_rope_scaling_factor": self.local_rope_scaling_factor,
                "global_rope_scaling_factor": self.global_rope_scaling_factor,
                "vision_encoder": None
                if self.vision_encoder is None
                else keras.layers.serialize(self.vision_encoder),
                "layer_norm_epsilon": self.layer_norm_epsilon,
                "dropout": self.dropout,
            }
        )
        return config

    def get_lora_target_names(self):
        target_names = super().get_lora_target_names()

        # Add these for `Gemma3VITAttention`.
        if not self.text_only_model:
            target_names += ["query_proj", "value_proj"]
        return target_names

    @classmethod
    def from_config(cls, config):
        config.update(
            {
                "vision_encoder": None
                if config["vision_encoder"] is None
                else keras.layers.deserialize(config["vision_encoder"]),
            }
        )

        return super().from_config(config)



import keras

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.preprocessing.masked_lm_mask_generator import (
    MaskedLMMaskGenerator,
)
from keras_hub.src.layers.preprocessing.multi_segment_packer import (
    MultiSegmentPacker,
)
from keras_hub.src.models.preprocessor import Preprocessor
from keras_hub.src.utils.tensor_utils import preprocessing_function


@keras_hub_export("keras_hub.models.MaskedLMPreprocessor")
class MaskedLMPreprocessor(Preprocessor):
    """Base class for masked language modeling preprocessing layers.

    `MaskedLMPreprocessor` tasks wrap a `keras_hub.tokenizer.Tokenizer` to
    create a preprocessing layer for masked language modeling tasks. It is
    intended to be paired with a `keras.models.MaskedLM` task.

    All `MaskedLMPreprocessor` take inputs a single input. This can be a single
    string, a batch of strings, or a tuple of batches of string segments that
    should be combined into a single sequence. See examples below. These inputs
    will be tokenized, combined, and masked randomly along the sequence.

    This layer will always output a `(x, y, sample_weight)` tuple, where `x`
    is a dictionary with the masked, tokenized inputs, `y` contains the tokens
    that were masked in `x`, and `sample_weight` marks where `y` contains padded
    values. The exact contents of `x` will vary depending on the model being
    used.

    All `MaskedLMPreprocessor` tasks include a `from_preset()` constructor
    which can be used to load a pre-trained config and vocabularies. You can
    call the `from_preset()` constructor directly on this base class, in which
    case the correct class for you model will be automatically instantiated.

    Examples.
    ```python
    preprocessor = keras_hub.models.MaskedLMPreprocessor.from_preset(
        "bert_base_en_uncased",
        sequence_length=256, # Optional.
    )

    # Tokenize, mask and pack a single sentence.
    x = "The quick brown fox jumped."
    x, y, sample_weight = preprocessor(x)

    # Preprocess a batch of labeled sentence pairs.
    first = ["The quick brown fox jumped.", "Call me Ishmael."]
    second = ["The fox tripped.", "Oh look, a whale."]
    x, y, sample_weight = preprocessor((first, second))

    # With a `tf.data.Dataset`.
    ds = tf.data.Dataset.from_tensor_slices((first, second))
    ds = ds.map(preprocessor, num_parallel_calls=tf.data.AUTOTUNE)
    ```
    """

    def __init__(
        self,
        tokenizer,
        sequence_length=512,
        truncate="round_robin",
        mask_selection_rate=0.15,
        mask_selection_length=96,
        mask_token_rate=0.8,
        random_token_rate=0.1,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.tokenizer = tokenizer
        self.packer = None
        self.sequence_length = sequence_length
        self.truncate = truncate
        self.mask_selection_rate = mask_selection_rate
        self.mask_selection_length = mask_selection_length
        self.mask_token_rate = mask_token_rate
        self.random_token_rate = random_token_rate
        self.masker = None

    def build(self, input_shape):
        super().build(input_shape)
        # Defer masker creation to `build()` so that we can be sure tokenizer
        # assets have loaded when restoring a saved model.
        self.packer = MultiSegmentPacker(
            start_value=self.tokenizer.start_token_id,
            end_value=self.tokenizer.end_token_id,
            pad_value=self.tokenizer.pad_token_id,
            truncate=self.truncate,
            sequence_length=self.sequence_length,
        )
        self.masker = MaskedLMMaskGenerator(
            mask_selection_rate=self.mask_selection_rate,
            mask_selection_length=self.mask_selection_length,
            mask_token_rate=self.mask_token_rate,
            random_token_rate=self.random_token_rate,
            vocabulary_size=self.tokenizer.vocabulary_size(),
            mask_token_id=self.tokenizer.mask_token_id,
            unselectable_token_ids=self.tokenizer.special_token_ids,
        )

    @preprocessing_function
    def call(self, x, y=None, sample_weight=None):
        x = x if isinstance(x, tuple) else (x,)
        x = tuple(self.tokenizer(segment) for segment in x)
        token_ids, segment_ids = self.packer(x)
        padding_mask = token_ids != self.tokenizer.pad_token_id
        masker_outputs = self.masker(token_ids)
        x = {
            "token_ids": masker_outputs["token_ids"],
            "padding_mask": padding_mask,
            "segment_ids": segment_ids,
            "mask_positions": masker_outputs["mask_positions"],
        }
        y = masker_outputs["mask_ids"]
        sample_weight = masker_outputs["mask_weights"]
        return keras.utils.pack_x_y_sample_weight(x, y, sample_weight)

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "sequence_length": self.sequence_length,
                "truncate": self.truncate,
                "mask_selection_rate": self.mask_selection_rate,
                "mask_selection_length": self.mask_selection_length,
                "mask_token_rate": self.mask_token_rate,
                "random_token_rate": self.random_token_rate,
            }
        )
        return config

    @property
    def sequence_length(self):
        """The padded length of model input sequences."""
        return self._sequence_length

    @sequence_length.setter
    def sequence_length(self, value):
        self._sequence_length = value
        if self.packer is not None:
            self.packer.sequence_length = value



from keras_hub.src.models.xception.xception_backbone import XceptionBackbone
from keras_hub.src.models.xception.xception_presets import backbone_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, XceptionBackbone)



import keras

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.bert.bert_backbone import BertBackbone
from keras_hub.src.models.bert.bert_backbone import bert_kernel_initializer
from keras_hub.src.models.bert.bert_text_classifier_preprocessor import (
    BertTextClassifierPreprocessor,
)
from keras_hub.src.models.text_classifier import TextClassifier


@keras_hub_export(
    [
        "keras_hub.models.BertTextClassifier",
        "keras_hub.models.BertClassifier",
    ]
)
class BertTextClassifier(TextClassifier):
    """An end-to-end BERT model for classification tasks.

    This model attaches a classification head to a
    `keras_hub.model.BertBackbone` instance, mapping from the backbone outputs
    to logits suitable for a classification task. For usage of this model with
    pre-trained weights, use the `from_preset()` constructor.

    This model can optionally be configured with a `preprocessor` layer, in
    which case it will automatically apply preprocessing to raw inputs during
    `fit()`, `predict()`, and `evaluate()`. This is done by default when
    creating the model with `from_preset()`.

    Disclaimer: Pre-trained models are provided on an "as is" basis, without
    warranties or conditions of any kind.

    Args:
        backbone: A `keras_hub.models.BertBackbone` instance.
        num_classes: int. Number of classes to predict.
        preprocessor: A `keras_hub.models.BertTextClassifierPreprocessor` or
            `None`. If `None`, this model will not apply preprocessing, and
            inputs should be preprocessed before calling the model.
        activation: Optional `str` or callable. The
            activation function to use on the model outputs. Set
            `activation="softmax"` to return output probabilities.
            Defaults to `None`.
        dropout: float. The dropout probability value, applied after the dense
            layer.

    Examples:

    Raw string data.
    ```python
    features = ["The quick brown fox jumped.", "I forgot my homework."]
    labels = [0, 3]

    # Pretrained classifier.
    classifier = keras_hub.models.BertTextClassifier.from_preset(
        "bert_base_en_uncased",
        num_classes=4,
    )
    classifier.fit(x=features, y=labels, batch_size=2)
    classifier.predict(x=features, batch_size=2)

    # Re-compile (e.g., with a new learning rate).
    classifier.compile(
        loss=keras.losses.SparseCategoricalCrossentropy(from_logits=True),
        optimizer=keras.optimizers.Adam(5e-5),
        jit_compile=True,
    )
    # Access backbone programmatically (e.g., to change `trainable`).
    classifier.backbone.trainable = False
    # Fit again.
    classifier.fit(x=features, y=labels, batch_size=2)
    ```

    Preprocessed integer data.
    ```python
    features = {
        "token_ids": np.ones(shape=(2, 12), dtype="int32"),
        "segment_ids": np.array([[0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0]] * 2),
        "padding_mask": np.array([[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0]] * 2),
    }
    labels = [0, 3]

    # Pretrained classifier without preprocessing.
    classifier = keras_hub.models.BertTextClassifier.from_preset(
        "bert_base_en_uncased",
        num_classes=4,
        preprocessor=None,
    )
    classifier.fit(x=features, y=labels, batch_size=2)
    ```

    Custom backbone and vocabulary.
    ```python
    features = ["The quick brown fox jumped.", "I forgot my homework."]
    labels = [0, 3]

    vocab = ["[UNK]", "[CLS]", "[SEP]", "[PAD]", "[MASK]"]
    vocab += ["The", "quick", "brown", "fox", "jumped", "."]
    tokenizer = keras_hub.models.BertTokenizer(
        vocabulary=vocab,
    )
    preprocessor = keras_hub.models.BertTextClassifierPreprocessor(
        tokenizer=tokenizer,
        sequence_length=128,
    )
    backbone = keras_hub.models.BertBackbone(
        vocabulary_size=30552,
        num_layers=4,
        num_heads=4,
        hidden_dim=256,
        intermediate_dim=512,
        max_sequence_length=128,
    )
    classifier = keras_hub.models.BertTextClassifier(
        backbone=backbone,
        preprocessor=preprocessor,
        num_classes=4,
    )
    classifier.fit(x=features, y=labels, batch_size=2)
    ```
    """

    backbone_cls = BertBackbone
    preprocessor_cls = BertTextClassifierPreprocessor

    def __init__(
        self,
        backbone,
        num_classes,
        preprocessor=None,
        activation=None,
        dropout=0.1,
        **kwargs,
    ):
        # === Layers ===
        self.backbone = backbone
        self.preprocessor = preprocessor
        self.output_dropout = keras.layers.Dropout(
            dropout,
            dtype=backbone.dtype_policy,
            name="classifier_dropout",
        )
        self.output_dense = keras.layers.Dense(
            num_classes,
            kernel_initializer=bert_kernel_initializer(),
            activation=activation,
            dtype=backbone.dtype_policy,
            name="logits",
        )

        # === Functional Model ===
        inputs = backbone.input
        pooled = backbone(inputs)["pooled_output"]
        pooled = self.output_dropout(pooled)
        outputs = self.output_dense(pooled)
        super().__init__(
            inputs=inputs,
            outputs=outputs,
            **kwargs,
        )

        # === Config ===
        self.num_classes = num_classes
        self.activation = keras.activations.get(activation)
        self.dropout = dropout

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "num_classes": self.num_classes,
                "activation": keras.activations.serialize(self.activation),
                "dropout": self.dropout,
            }
        )
        return config



import keras

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.preprocessing.multi_segment_packer import (
    MultiSegmentPacker,
)
from keras_hub.src.models.text_classifier_preprocessor import (
    TextClassifierPreprocessor,
)
from keras_hub.src.models.xlm_roberta.xlm_roberta_backbone import (
    XLMRobertaBackbone,
)
from keras_hub.src.models.xlm_roberta.xlm_roberta_tokenizer import (
    XLMRobertaTokenizer,
)
from keras_hub.src.utils.tensor_utils import preprocessing_function


@keras_hub_export(
    [
        "keras_hub.models.XLMRobertaTextClassifierPreprocessor",
        "keras_hub.models.XLMRobertaPreprocessor",
    ]
)
class XLMRobertaTextClassifierPreprocessor(TextClassifierPreprocessor):
    """An XLM-RoBERTa preprocessing layer which tokenizes and packs inputs.

    This preprocessing layer will do three things:

    1. Tokenize any number of input segments using the `tokenizer`.
    2. Pack the inputs together using a `keras_hub.layers.MultiSegmentPacker`.
      with the appropriate `"<s>"`, `"</s>"` and `"<pad>"` tokens, i.e., adding
      a single `"<s>"` at the start of the entire sequence, `"</s></s>"` at the
      end of each segment, save the last and a `"</s>"` at the end of the
      entire sequence.
    3. Construct a dictionary with keys `"token_ids"` and `"padding_mask"`,
      that can be passed directly to an XLM-RoBERTa model.

    This layer can be used directly with `tf.data.Dataset.map` to preprocess
    string data in the `(x, y, sample_weight)` format used by
    `keras.Model.fit`.

    Args:
        tokenizer: A `keras_hub.tokenizers.XLMRobertaTokenizer` instance.
        sequence_length: The length of the packed inputs.
        truncate: The algorithm to truncate a list of batched segments to fit
            within `sequence_length`. The value can be either `round_robin` or
            `waterfall`:
                - `"round_robin"`: Available space is assigned one token at a
                    time in a round-robin fashion to the inputs that still need
                    some, until the limit is reached.
                - `"waterfall"`: The allocation of the budget is done using a
                    "waterfall" algorithm that allocates quota in a
                    left-to-right manner and fills up the buckets until we run
                    out of budget. It supports an arbitrary number of segments.

    Call arguments:
        x: A tensor of single string sequences, or a tuple of multiple
            tensor sequences to be packed together. Inputs may be batched or
            unbatched. For single sequences, raw python inputs will be converted
            to tensors. For multiple sequences, pass tensors directly.
        y: Any label data. Will be passed through unaltered.
        sample_weight: Any label weight data. Will be passed through unaltered.

    Examples:

    Directly calling the layer on data.
    ```python
    preprocessor = keras_hub.models.TextClassifierPreprocessor.from_preset(
        "xlm_roberta_base_multi"
    )

    # Tokenize and pack a single sentence.
    preprocessor("The quick brown fox jumped.")

    # Tokenize a batch of single sentences.
    preprocessor(["The quick brown fox jumped.", "اسمي اسماعيل"])

    # Preprocess a batch of sentence pairs.
    # When handling multiple sequences, always convert to tensors first!
    first = tf.constant(["The quick brown fox jumped.", "اسمي اسماعيل"])
    second = tf.constant(["The fox tripped.", "الأسد ملك الغابة"])
    preprocessor((first, second))

    # Custom vocabulary.
    def train_sentencepiece(ds, vocab_size):
        bytes_io = io.BytesIO()
        sentencepiece.SentencePieceTrainer.train(
            sentence_iterator=ds.as_numpy_iterator(),
            model_writer=bytes_io,
            vocab_size=vocab_size,
            model_type="WORD",
            unk_id=0,
            bos_id=1,
            eos_id=2,
        )
        return bytes_io.getvalue()
    ds = tf.data.Dataset.from_tensor_slices(
        ["the quick brown fox", "the earth is round"]
    )
    proto = train_sentencepiece(ds, vocab_size=10)
    tokenizer = keras_hub.models.XLMRobertaTokenizer(proto=proto)
    preprocessor = keras_hub.models.XLMRobertaTextClassifierPreprocessor(
        tokenizer
    )
    preprocessor("The quick brown fox jumped.")
    ```

    Mapping with `tf.data.Dataset`.
    ```python
    preprocessor = keras_hub.models.TextClassifierPreprocessor.from_preset(
        "xlm_roberta_base_multi"
    )

    first = tf.constant(["The quick brown fox jumped.", "Call me Ishmael."])
    second = tf.constant(["The fox tripped.", "Oh look, a whale."])
    label = tf.constant([1, 1])

    # Map labeled single sentences.
    ds = tf.data.Dataset.from_tensor_slices((first, label))
    ds = ds.map(preprocessor, num_parallel_calls=tf.data.AUTOTUNE)

    # Map unlabeled single sentences.
    ds = tf.data.Dataset.from_tensor_slices(first)
    ds = ds.map(preprocessor, num_parallel_calls=tf.data.AUTOTUNE)

    # Map labeled sentence pairs.
    ds = tf.data.Dataset.from_tensor_slices(((first, second), label))
    ds = ds.map(preprocessor, num_parallel_calls=tf.data.AUTOTUNE)

    # Map unlabeled sentence pairs.
    ds = tf.data.Dataset.from_tensor_slices((first, second))
    # Watch out for tf.data's default unpacking of tuples here!
    # Best to invoke the `preprocessor` directly in this case.
    ds = ds.map(
        lambda first, second: preprocessor(x=(first, second)),
        num_parallel_calls=tf.data.AUTOTUNE,
    )
    ```
    """

    backbone_cls = XLMRobertaBackbone
    tokenizer_cls = XLMRobertaTokenizer

    def build(self, input_shape):
        # Roberta is doubles up the sep token, so we override build.
        self.packer = MultiSegmentPacker(
            start_value=self.tokenizer.start_token_id,
            end_value=self.tokenizer.end_token_id,
            sep_value=[self.tokenizer.end_token_id] * 2,
            pad_value=self.tokenizer.pad_token_id,
            truncate=self.truncate,
            sequence_length=self.sequence_length,
        )
        self.built = True

    @preprocessing_function
    def call(self, x, y=None, sample_weight=None):
        output = super().call(x, y=y, sample_weight=sample_weight)
        x, y, sample_weight = keras.utils.unpack_x_y_sample_weight(output)
        # Backbone has no segment ID input.
        del x["segment_ids"]
        return keras.utils.pack_x_y_sample_weight(x, y, sample_weight)



import numpy as np

from keras_hub.src.models.cspnet.cspnet_backbone import CSPNetBackbone

backbone_cls = CSPNetBackbone


def convert_backbone_config(timm_config):
    timm_architecture = timm_config["architecture"]

    if timm_architecture == "cspdarknet53":
        stem_filters = 32
        stem_kernel_size = 3
        stem_strides = 1
        stackwise_depth = [1, 2, 8, 8, 4]
        stackwise_num_filters = [64, 128, 256, 512, 1024]
        bottle_ratio = (0.5,) + (1.0,)
        block_ratio = (1.0,) + (0.5,)
        expand_ratio = (2.0,) + (1.0,)
        stem_padding = "same"
        stem_pooling = None
        stage_type = "csp"
        groups = 1
        block_type = "dark_block"
        down_growth = True
        stackwise_strides = [2, 2, 2, 2, 2]
        avg_down = False
        cross_linear = False
    elif timm_architecture == "cspresnet50":
        stem_filters = 64
        stem_kernel_size = 7
        stem_strides = 4
        stackwise_depth = [3, 3, 5, 2]
        stackwise_strides = [1, 2, 2, 2]
        stackwise_num_filters = [128, 256, 512, 1024]
        block_type = "bottleneck_block"
        stage_type = "csp"
        bottle_ratio = [0.5]
        block_ratio = [1.0]
        expand_ratio = [2.0]
        stem_padding = "valid"
        stem_pooling = "max"
        avg_down = False
        groups = 1
        down_growth = False
        cross_linear = True
    elif timm_architecture == "cspresnext50":
        stem_filters = 64
        stem_kernel_size = 7
        stem_strides = 4
        stackwise_depth = [3, 3, 5, 2]
        stackwise_num_filters = [256, 512, 1024, 2048]
        bottle_ratio = [1.0]
        block_ratio = [0.5]
        expand_ratio = [1.0]
        stage_type = "csp"
        block_type = "bottleneck_block"
        stem_pooling = "max"
        stackwise_strides = [1, 2, 2, 2]
        groups = 32
        stem_padding = "valid"
        avg_down = False
        down_growth = False
        cross_linear = True
    elif timm_architecture == "darknet53":
        stem_filters = 32
        stem_kernel_size = 3
        stem_strides = 1
        stackwise_depth = [1, 2, 8, 8, 4]
        stackwise_num_filters = [64, 128, 256, 512, 1024]
        bottle_ratio = [0.5]
        block_ratio = [1.0]
        groups = 1
        expand_ratio = [1.0]
        stage_type = "dark"
        block_type = "dark_block"
        stem_pooling = None
        stackwise_strides = [2, 2, 2, 2, 2]
        stem_padding = "same"
        avg_down = False
        down_growth = False
        cross_linear = False
    else:
        raise ValueError(
            f"Currently, the architecture {timm_architecture} is not supported."
        )
    return dict(
        stem_filters=stem_filters,
        stem_kernel_size=stem_kernel_size,
        stem_strides=stem_strides,
        stackwise_depth=stackwise_depth,
        stackwise_num_filters=stackwise_num_filters,
        bottle_ratio=bottle_ratio,
        block_ratio=block_ratio,
        expand_ratio=expand_ratio,
        stage_type=stage_type,
        block_type=block_type,
        stackwise_strides=stackwise_strides,
        down_growth=down_growth,
        stem_pooling=stem_pooling,
        stem_padding=stem_padding,
        avg_down=avg_down,
        cross_linear=cross_linear,
        groups=groups,
    )


def convert_weights(backbone, loader, timm_config):
    def port_conv2d(hf_weight_prefix, keras_layer_name):
        loader.port_weight(
            backbone.get_layer(keras_layer_name).kernel,
            hf_weight_key=f"{hf_weight_prefix}.weight",
            hook_fn=lambda x, _: np.transpose(x, (2, 3, 1, 0)),
        )

    def port_batch_normalization(hf_weight_prefix, keras_layer_name):
        loader.port_weight(
            backbone.get_layer(keras_layer_name).gamma,
            hf_weight_key=f"{hf_weight_prefix}.weight",
        )
        loader.port_weight(
            backbone.get_layer(keras_layer_name).beta,
            hf_weight_key=f"{hf_weight_prefix}.bias",
        )
        loader.port_weight(
            backbone.get_layer(keras_layer_name).moving_mean,
            hf_weight_key=f"{hf_weight_prefix}.running_mean",
        )
        loader.port_weight(
            backbone.get_layer(keras_layer_name).moving_variance,
            hf_weight_key=f"{hf_weight_prefix}.running_var",
        )

    # Stem

    stem_filter = backbone.stem_filters
    if not isinstance(stem_filter, (tuple, list)):
        stem_filter = [stem_filter]

    for i in range(len(stem_filter)):
        port_conv2d(f"stem.conv{i + 1}.conv", f"csp_stem_conv_{i}")
        port_batch_normalization(f"stem.conv{i + 1}.bn", f"csp_stem_bn_{i}")

    # Stages
    stackwise_depth = backbone.stackwise_depth
    stage_type = backbone.stage_type
    block_type = backbone.block_type
    strides = backbone.stackwise_strides

    for idx, block in enumerate(stackwise_depth):
        if strides[idx] != 1 or stage_type == "dark":
            if strides[idx] == 2 and backbone.avg_down:
                port_conv2d(
                    f"stages.{idx}.conv_down.1.conv",
                    f"stage_{idx}_{stage_type}_conv_down_1",
                )
                port_batch_normalization(
                    f"stages.{idx}.conv_down.1.bn",
                    f"stage_{idx}_{stage_type}_bn_1",
                )
            else:
                port_conv2d(
                    f"stages.{idx}.conv_down.conv",
                    f"stage_{idx}_{stage_type}_conv_down_1",
                )
                port_batch_normalization(
                    f"stages.{idx}.conv_down.bn",
                    f"stage_{idx}_{stage_type}_bn_1",
                )
        if stage_type != "dark":
            port_conv2d(
                f"stages.{idx}.conv_exp.conv",
                f"stage_{idx}_{stage_type}_conv_exp",
            )
            port_batch_normalization(
                f"stages.{idx}.conv_exp.bn", f"stage_{idx}_{stage_type}_bn_2"
            )

        for i in range(block):
            port_conv2d(
                f"stages.{idx}.blocks.{i}.conv1.conv",
                f"stage_{idx}_block_{i}_{block_type}_conv_1",
            )
            port_batch_normalization(
                f"stages.{idx}.blocks.{i}.conv1.bn",
                f"stage_{idx}_block_{i}_{block_type}_bn_1",
            )
            port_conv2d(
                f"stages.{idx}.blocks.{i}.conv2.conv",
                f"stage_{idx}_block_{i}_{block_type}_conv_2",
            )
            port_batch_normalization(
                f"stages.{idx}.blocks.{i}.conv2.bn",
                f"stage_{idx}_block_{i}_{block_type}_bn_2",
            )
            if block_type == "bottleneck_block":
                port_conv2d(
                    f"stages.{idx}.blocks.{i}.conv3.conv",
                    f"stage_{idx}_block_{i}_{block_type}_conv_3",
                )
                port_batch_normalization(
                    f"stages.{idx}.blocks.{i}.conv3.bn",
                    f"stage_{idx}_block_{i}_{block_type}_bn_3",
                )

        if stage_type == "csp":
            port_conv2d(
                f"stages.{idx}.conv_transition_b.conv",
                f"stage_{idx}_{stage_type}_conv_transition_b",
            )
            port_batch_normalization(
                f"stages.{idx}.conv_transition_b.bn",
                f"stage_{idx}_{stage_type}_transition_b_bn",
            )

        if stage_type != "dark":
            port_conv2d(
                f"stages.{idx}.conv_transition.conv",
                f"stage_{idx}_{stage_type}_conv_transition",
            )
            port_batch_normalization(
                f"stages.{idx}.conv_transition.bn",
                f"stage_{idx}_{stage_type}_transition_bn",
            )


def convert_head(task, loader, timm_config):
    loader.port_weight(
        task.output_dense.kernel,
        hf_weight_key="head.fc.weight",
        hook_fn=lambda x, _: np.transpose(np.squeeze(x)),
    )
    loader.port_weight(
        task.output_dense.bias,
        hf_weight_key="head.fc.bias",
    )
